from typing import Any, Callable

from kagglehub import registry
from kagglehub.handle import ResourceHandle
from kagglehub.resolver import Resolver
from tests.fixtures import BaseTestCase

SOME_VALUE: str = "test"


class FakeHandle(ResourceHandle):
    def to_url(self) -> str:
        raise NotImplementedError()


class FakeImpl(Resolver[FakeHandle]):
    def __init__(
        self,
        is_supported_fn: Callable[[FakeHandle], bool],
        call_fn: Callable[[FakeHandle], str],
    ):
        self._is_supported_fn = is_supported_fn
        self._call_fn = call_fn

    def is_supported(self, *args: Any, **kwargs: Any) -> bool:  # noqa: ANN401
        return self._is_supported_fn(*args, **kwargs)

    def __call__(self, *args: Any, **kwargs: Any) -> str:  # noqa: ANN401
        return self._call_fn(*args, **kwargs)


def fail_fn(*_, **__) -> str:  # noqa: ANN002, ANN003
    msg = "fail_fn should not be called"
    raise AssertionError(msg)


class RegistryTest(BaseTestCase):
    def test_calls_only_supported(self) -> None:
        r = registry.MultiImplRegistry[FakeHandle]("test")
        r.add_implementation(FakeImpl(lambda *_, **__: True, lambda *_, **__: SOME_VALUE))
        r.add_implementation(FakeImpl(lambda *_, **__: False, fail_fn))

        val = r(FakeHandle())

        self.assertEqual(SOME_VALUE, val)

    def test_calls_first_supported_reverse(self) -> None:
        r = registry.MultiImplRegistry[FakeHandle]("test")
        r.add_implementation(FakeImpl(lambda *_, **__: True, fail_fn))
        r.add_implementation(FakeImpl(lambda *_, **__: True, lambda *_, **__: SOME_VALUE))

        val = r(FakeHandle())

        self.assertEqual(SOME_VALUE, val)

    def test_calls_throw_not_supported(self) -> None:
        r = registry.MultiImplRegistry[FakeHandle]("test")
        r.add_implementation(FakeImpl(lambda *_, **__: False, fail_fn))

        self.assertRaisesRegex(RuntimeError, r"Missing implementation", r, SOME_VALUE)



import os
import unittest
from collections.abc import Generator
from contextlib import contextmanager
from tempfile import TemporaryDirectory
from unittest import mock

from kagglehub.config import (
    CACHE_FOLDER_ENV_VAR_NAME,
    CREDENTIALS_FOLDER_ENV_VAR_NAME,
    KEY_ENV_VAR_NAME,
    USERNAME_ENV_VAR_NAME,
)


@contextmanager
def create_test_cache() -> Generator[str, None, None]:
    with TemporaryDirectory() as d:
        with mock.patch.dict(os.environ, {CACHE_FOLDER_ENV_VAR_NAME: d}):
            yield d


def list_files_recursively(path: str) -> list[str]:
    """List all files recursively in the given path.
    If the path is a file, return a list containing only that file.
    If the path is a directory, list all files recursively in that directory."""
    if os.path.isfile(path):
        return [os.path.basename(path)]

    files = []
    for root, _, filenames in os.walk(path):
        for filename in filenames:
            # Create a relative path from the directory and add it to the list
            rel_file = os.path.relpath(os.path.join(root, filename), path)
            if not rel_file.startswith("./?"):
                files.append(rel_file)
    return sorted(files)


def list_columns(path: str) -> list[str]:
    """Assuming the path is a CSV, list all columns sorted lexicographically"""
    with open(path) as file:
        first_line = file.readline().strip()
        return sorted(first_line.split(","))


def assert_files(test_case: unittest.TestCase, path: str, expected_files: list[str]) -> None:
    """Assert that all expected files exist and are non-empty."""
    files = list_files_recursively(path)
    expected_files_sorted = sorted(expected_files)
    test_case.assertEqual(files, expected_files_sorted)

    # Assert that each file's size is greater than zero
    for file in files:
        if os.path.isfile(path):
            file_path = os.path.join(os.path.dirname(path), file)
        else:
            file_path = os.path.join(path, file)

        test_case.assertGreater(os.path.getsize(file_path), 0, f"File {file} is empty")


def assert_columns(test_case: unittest.TestCase, path: str, expected_columns: list[str]) -> None:
    """Assert that the given path to a CSV has the expected columns."""
    columns = list_columns(path)
    test_case.assertEqual(columns, sorted(expected_columns))


@contextmanager
def unauthenticated() -> Generator[None, None, None]:
    with mock.patch.dict(
        os.environ,
        {USERNAME_ENV_VAR_NAME: "", KEY_ENV_VAR_NAME: "", CREDENTIALS_FOLDER_ENV_VAR_NAME: "/nonexistent"},
    ):
        yield



from typing import Any, Optional

# WARNING: This module is intended to be imported only at runtime, with
# specific error handling to inform users that they need to install the
# appropriate `hf-datasets` extras in order to use these methods. Adding
# any new dependencies here must correspond with an addition to the
# list of `hf-datasets` optional dependencies in pyproject.toml.
from datasets import Dataset

from kagglehub.pandas_datasets import load_pandas_dataset

MULTIPLE_DATA_FRAMES_ERROR_MESSAGE = (
    "Loading a Huggingface dataset requires the production of exactly one DataFrame. "
    "For example, if using an Excel-based dataset, you must specify a single sheet"
)
DEFAULT_PANDAS_KWARGS = {"preserve_index": False}


def load_hf_dataset(
    handle: str,
    path: str,
    *,
    pandas_kwargs: Any = None,  # noqa: ANN401
    sql_query: Optional[str] = None,
    hf_kwargs: Any = None,  # noqa: ANN401
) -> Dataset:
    """Load a Kaggle Dataset into a Huggingface Dataset (via pandas)

    Args:
        handle: (string) The dataset handle
        path: (string) Path to a file within the dataset
        pandas_kwargs:
            (dict) Optional set of kwargs to pass to the pandas `read_*` method while constructing the DataFrame(s)
        sql_query:
            (string) Argument to be used for SQLite files. Required when reading a SQLite file. See pandas documentation
            for details: https://pandas.pydata.org/docs/reference/api/pandas.read_sql_query.html
        hf_kwargs:
            (dict) Optional set of kwargs to pass to Dataset.from_pandas() while constructing the Dataset
    Returns:
        Dataset
    """
    hf_kwargs = {} if hf_kwargs is None else hf_kwargs
    pandas_kwargs = {} if pandas_kwargs is None else pandas_kwargs
    result = load_pandas_dataset(handle, path, pandas_kwargs=pandas_kwargs, sql_query=sql_query)

    # We're only loading one Dataset at a time, so alert the user if their args produce multiple. This will
    # probably only happen if the user is trying to load an Excel-based file with many sheets and none specified
    if isinstance(result, dict) or isinstance(result, list):
        raise ValueError(MULTIPLE_DATA_FRAMES_ERROR_MESSAGE)

    # NOTE: Order matters here as we're letting users override our specified defaults, namely preserve_index=False.
    # This may be valuable in the edge case that a user does actually want the index persisted as a column.
    merged_kwargs = {**DEFAULT_PANDAS_KWARGS, **hf_kwargs}
    return Dataset.from_pandas(result, **merged_kwargs)



import hashlib
import mimetypes
import os
from collections.abc import Generator
from contextlib import contextmanager
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any
from unittest import mock

from flask import Flask, Response
from flask.typing import ResponseReturnValue

from kagglehub.config import CACHE_FOLDER_ENV_VAR_NAME, get_kaggle_api_endpoint
from kagglehub.handle import ResourceHandle
from kagglehub.integrity import GCS_HASH_HEADER, to_b64_digest

MOCK_GCS_BUCKET_BASE_PATH = "/mock-gcs-bucket/file-path"
AUTO_COMPRESSED_FILE_NAME = "shapes.csv"
LOCATION_HEADER = "Location"
CONTENT_LENGTH_HEADER = "Content-Length"


def get_test_file_path(relative_path: str) -> str:
    return os.path.join(Path(__file__).parent, "data", relative_path)


def resolve_endpoint(var_name: str = "KAGGLE_API_ENDPOINT") -> tuple[str, int]:
    endpoint = os.environ.get(var_name, "127.0.0.1:7777")
    address, port = endpoint.replace("http://", "").split(":")
    return address, int(port)


def get_mocked_gcs_signed_url(file_name: str) -> str:
    return f"{get_kaggle_api_endpoint()}{MOCK_GCS_BUCKET_BASE_PATH}/{file_name}?X-Goog-Headers=all-kinds-of-stuff"


# All downloads, regardless of archive or file, happen via GCS signed URLs. We mock the 302 and handle
# the redirect not only to be thorough--without this, the response.url in download_file (clients.py)
# will not pick up on followed redirect URL being different from the originally requested URL.
def get_gcs_redirect_response(file_name: str) -> ResponseReturnValue:
    return (
        Response(
            headers={
                LOCATION_HEADER: get_mocked_gcs_signed_url(file_name),
                CONTENT_LENGTH_HEADER: "0",
            }
        ),
        302,
    )


@contextmanager
def create_test_cache() -> Generator[str, None, None]:
    with TemporaryDirectory() as d:
        with mock.patch.dict(os.environ, {CACHE_FOLDER_ENV_VAR_NAME: d}):
            yield d


class InvalidResourceHandle(ResourceHandle):
    def __init__(self):
        self.owner = "invalid"

    def to_url(self) -> str:
        return "invalid"


def add_mock_gcs_route(app: Flask) -> None:
    """Adds the mock GCS route for handling signed URL redirects"""

    app.add_url_rule(
        f"{MOCK_GCS_BUCKET_BASE_PATH}/<file_name>",
        endpoint="handle_mock_gcs_redirect",
        view_func=handle_mock_gcs_redirect,
        methods=["get"],
    )


def handle_mock_gcs_redirect(file_name: str) -> ResponseReturnValue:
    test_file_path = get_test_file_path(file_name)

    def generate_file_content() -> Generator[bytes, Any, None]:
        with open(test_file_path, "rb") as f:
            while True:
                chunk = f.read(4096)  # Read file in chunks
                if not chunk:
                    break
                yield chunk

    with open(test_file_path, "rb") as f:
        content = f.read()
        file_hash = hashlib.md5()
        file_hash.update(content)
        return (
            Response(
                generate_file_content(),
                headers={
                    GCS_HASH_HEADER: f"md5={to_b64_digest(file_hash)}",
                    "Content-Length": str(os.path.getsize(test_file_path)),
                    "Content-Type": mimetypes.guess_type(test_file_path)[0] or "application/octet-stream",
                },
            ),
            200,
        )



import logging
from typing import Any, Optional, Union

from kagglehub import registry
from kagglehub.datasets_enums import KaggleDatasetAdapter
from kagglehub.datasets_helpers import create_dataset_or_version
from kagglehub.gcs_upload import normalize_patterns, upload_files_and_directories
from kagglehub.handle import parse_dataset_handle
from kagglehub.logger import EXTRA_CONSOLE_BLOCK

logger = logging.getLogger(__name__)


# Patterns that are always ignored for dataset uploading.
DEFAULT_IGNORE_PATTERNS = [".git/", "*/.git/", ".cache/", ".huggingface/"]
# Mapping of adapters to the optional dependencies required to run them
LOAD_DATASET_ADAPTER_OPTIONAL_DEPENDENCIES_MAP = {
    KaggleDatasetAdapter.HUGGING_FACE: "hf-datasets",
    KaggleDatasetAdapter.PANDAS: "pandas-datasets",
}


def dataset_download(handle: str, path: Optional[str] = None, *, force_download: Optional[bool] = False) -> str:
    """Download dataset files
    Args:
        handle: (string) the dataset handle
        path: (string) Optional path to a file within a dataset
        force_download: (bool) Optional flag to force download a dataset, even if it's cached
    Returns:
        A string requesting the path to the requested dataset files.
    """
    h = parse_dataset_handle(handle)
    logger.info(f"Downloading Dataset: {h.to_url()} ...", extra={**EXTRA_CONSOLE_BLOCK})
    return registry.dataset_resolver(h, path, force_download=force_download)


def dataset_upload(
    handle: str,
    local_dataset_dir: str,
    version_notes: str = "",
    ignore_patterns: Optional[Union[list[str], str]] = None,
) -> None:
    """Upload dataset files.
    Args:
        handle: (string) the dataset handle.
        local_dataset_dir: (string) path to a file in a local directory.
        version_notes: (string) Optional to write dataset versions.
        ignore_patterns (str or list[str], optional):
            Additional ignore patterns to DEFAULT_IGNORE_PATTERNS.
            Files matching any of the patterns are not uploaded.
            Patterns are standard wildcards that can be matched by
            https://docs.python.org/3/library/fnmatch.html.
            Use a pattern ending with "/" to ignore the whole dir,
            e.g., ".git/" is equivalent to ".git/*".
    """
    h = parse_dataset_handle(handle)
    logger.info(f"Uploading Dataset {h.to_url()} ...")
    if h.is_versioned():
        is_versioned_exception = "The dataset handle should not include the version"
        raise ValueError(is_versioned_exception)

    tokens = upload_files_and_directories(
        local_dataset_dir,
        item_type="dataset",
        ignore_patterns=normalize_patterns(default=DEFAULT_IGNORE_PATTERNS, additional=ignore_patterns),
    )

    create_dataset_or_version(h, tokens, version_notes)


def load_dataset(
    adapter: KaggleDatasetAdapter,
    # In the form of {owner_slug}/{dataset_slug} or {owner_slug}/{dataset_slug}/versions/{version_number}
    handle: str,
    path: str,
    *,
    pandas_kwargs: Any = None,  # noqa: ANN401
    sql_query: Optional[str] = None,
    hf_kwargs: Any = None,  # noqa: ANN401
) -> Any:  # noqa: ANN401
    """Load a Kaggle Dataset into a python object based on the selected adapter

    Args:
        adapter: (KaggleDatasetAdapter) The adapter used to load the dataset
        handle: (string) The dataset handle
        path: (string) Path to a file within the dataset
        pandas_kwargs:
            (dict) Optional set of kwargs to pass to the pandas `read_*` method while constructing the DataFrame(s)
        sql_query:
            (string) Argument to be used for SQLite files. Required when reading a SQLite file. See pandas documentation
            for details: https://pandas.pydata.org/docs/reference/api/pandas.read_sql_query.html
        hf_kwargs:
            (dict) Optional set of kwargs to pass to Dataset.from_pandas() while constructing the Dataset
    Returns:
        A python object based on the selected adapter:
            - 'pandas': A DataFrame (or dict[int | str, DataFrame] for Excel-like files with multiple sheets)
            - 'hugging_face': A Huggingface Dataset (via pandas)
    """
    try:
        if adapter is KaggleDatasetAdapter.HUGGING_FACE:
            import kagglehub.hf_datasets

            return kagglehub.hf_datasets.load_hf_dataset(
                handle,
                path,
                pandas_kwargs=pandas_kwargs,
                sql_query=sql_query,
                hf_kwargs=hf_kwargs,
            )
        elif adapter is KaggleDatasetAdapter.PANDAS:
            import kagglehub.pandas_datasets

            return kagglehub.pandas_datasets.load_pandas_dataset(
                handle, path, pandas_kwargs=pandas_kwargs, sql_query=sql_query
            )
        else:
            not_implemented_error_message = f"{adapter} is not yet implemented"
            raise NotImplementedError(not_implemented_error_message)
    except ImportError:
        adapter_optional_dependency = LOAD_DATASET_ADAPTER_OPTIONAL_DEPENDENCIES_MAP[adapter]
        import_warning_message = (
            f"The 'load_dataset' function requires the '{adapter_optional_dependency}' extras. "
            f"Install them with 'pip install kagglehub[{adapter_optional_dependency}]'"
        )
        # Inform the user if we detect that they didn't install everything
        raise ImportError(import_warning_message) from None



"""Functions to parse resource handles."""

import abc
from dataclasses import asdict, dataclass
from typing import Optional

from kagglehub.config import get_kaggle_api_endpoint

NUM_VERSIONED_DATASET_PARTS = 4  # e.g.: <owner>/<dataset>/versions/<version>
NUM_UNVERSIONED_DATASET_PARTS = 2  # e.g.: <owner>/<dataset>

NUM_VERSIONED_MODEL_PARTS = 5  # e.g.: <owner>/<model>/<framework>/<variation>/<version>
NUM_UNVERSIONED_MODEL_PARTS = 4  # e.g.: <owner>/<model>/<framework>/<variation>

NUM_UNVERSIONED_NOTEBOOK_PARTS = 2  # e.g.: <owner>/<notebook>
NUM_VERSIONED_NOTEBOOK_PARTS = 4  # e.g.: <owner>/<notebook>/versions/<version>


@dataclass
class ResourceHandle:
    @abc.abstractmethod
    def to_url(self) -> str:
        """Returns URL to the resource detail page."""
        pass


@dataclass
class ModelHandle(ResourceHandle):
    owner: str
    model: str
    framework: str
    variation: str
    version: Optional[int]

    def is_versioned(self) -> bool:
        return self.version is not None and self.version > 0

    def __str__(self) -> str:
        handle_str = f"{self.owner}/{self.model}/{self.framework}/{self.variation}"
        if self.is_versioned():
            return f"{handle_str}/{self.version}"
        return handle_str

    def to_url(self) -> str:
        endpoint = get_kaggle_api_endpoint()
        if self.is_versioned():
            return f"{endpoint}/models/{self.owner}/{self.model}/{self.framework}/{self.variation}/{self.version}"
        else:
            return f"{endpoint}/models/{self.owner}/{self.model}/{self.framework}/{self.variation}"


@dataclass
class DatasetHandle(ResourceHandle):
    owner: str
    dataset: str
    version: Optional[int] = None

    def is_versioned(self) -> bool:
        return self.version is not None and self.version > 0

    def __str__(self) -> str:
        handle_str = f"{self.owner}/{self.dataset}"
        if self.is_versioned():
            return f"{handle_str}/versions/{self.version}"
        return handle_str

    def to_url(self) -> str:
        endpoint = get_kaggle_api_endpoint()
        base_url = f"{endpoint}/datasets/{self.owner}/{self.dataset}"
        if self.is_versioned():
            return f"{base_url}/versions/{self.version}"
        return base_url


@dataclass
class CompetitionHandle(ResourceHandle):
    competition: str

    def __str__(self) -> str:
        handle_str = f"{self.competition}"
        return handle_str

    def to_url(self) -> str:
        endpoint = get_kaggle_api_endpoint()
        base_url = f"{endpoint}/competitions/{self.competition}"
        return base_url


@dataclass
class NotebookHandle(ResourceHandle):
    owner: str
    notebook: str
    version: Optional[int] = None

    def is_versioned(self) -> bool:
        return self.version is not None and self.version > 0

    def __str__(self) -> str:
        handle_str = f"{self.owner}/{self.notebook}"
        if self.is_versioned():
            return f"{handle_str}/versions/{self.version}"
        return handle_str

    def to_url(self) -> str:
        endpoint = get_kaggle_api_endpoint()
        base_url = f"{endpoint}/code/{self.owner}/{self.notebook}"
        if self.is_versioned():
            return f"{base_url}/versions/{self.version}"
        return base_url


class UtilityScriptHandle(NotebookHandle):
    pass


def parse_dataset_handle(handle: str) -> DatasetHandle:
    parts = handle.split("/")

    if len(parts) == NUM_VERSIONED_DATASET_PARTS:
        # Versioned handle
        # e.g.: <owner>/>dataset>/versions/<version>
        try:
            version = int(parts[3])
        except ValueError as err:
            msg = f"Invalid version number: {parts[3]}"
            raise ValueError(msg) from err
        return DatasetHandle(
            owner=parts[0],
            dataset=parts[1],
            version=version,
        )
    elif len(parts) == NUM_UNVERSIONED_DATASET_PARTS:
        # Unversioned handle
        # e.g.: <owner>/<dataset>
        return DatasetHandle(
            owner=parts[0],
            dataset=parts[1],
            version=None,
        )

    msg = f"Invalid dataset handle: {handle}"
    raise ValueError(msg)


def parse_model_handle(handle: str) -> ModelHandle:
    parts = handle.split("/")

    if len(parts) == NUM_VERSIONED_MODEL_PARTS:
        # Versioned handle
        # e.g.: <owner>/<model>/<framework>/<variation>/<version>
        try:
            version = int(parts[4])
        except ValueError as err:
            msg = f"Invalid version number: {parts[4]}"
            raise ValueError(msg) from err

        return ModelHandle(
            owner=parts[0],
            model=parts[1],
            framework=parts[2],
            variation=parts[3],
            version=version,
        )
    elif len(parts) == NUM_UNVERSIONED_MODEL_PARTS:
        # Unversioned handle
        # e.g.: <owner>/<model>/<framework>/<variation>
        return ModelHandle(
            owner=parts[0],
            model=parts[1],
            framework=parts[2],
            variation=parts[3],
            version=None,
        )

    msg = f"Invalid model handle: {handle}"
    raise ValueError(msg)


def parse_competition_handle(handle: str) -> CompetitionHandle:
    if "/" in handle:
        msg = f"Invalid competition handle: {handle}"
        raise ValueError(msg)

    return CompetitionHandle(competition=handle)


def parse_notebook_handle(handle: str) -> NotebookHandle:
    parts = handle.split("/")

    if len(parts) == NUM_VERSIONED_NOTEBOOK_PARTS:
        # Versioned handle
        # e.g.: <owner>/<notebook>/versions/<version>
        try:
            version = int(parts[3])
        except ValueError as err:
            msg = f"Invalid version number: {parts[3]}"
            raise ValueError(msg) from err
        return NotebookHandle(
            owner=parts[0],
            notebook=parts[1],
            version=version,
        )

    elif len(parts) == NUM_UNVERSIONED_NOTEBOOK_PARTS:
        # Unversioned handle
        # e.g.: <owner>/<notebook>
        return NotebookHandle(
            owner=parts[0],
            notebook=parts[1],
            version=None,
        )

    msg = f"Invalid notebook handle: {handle}"
    raise ValueError(msg)


def parse_utility_script_handle(handle: str) -> UtilityScriptHandle:
    notebook_handle = parse_notebook_handle(handle)
    return UtilityScriptHandle(**asdict(notebook_handle))



import os
from unittest import mock

import requests

import kagglehub
from kagglehub.config import DISABLE_COLAB_CACHE_ENV_VAR_NAME, TBE_RUNTIME_ADDR_ENV_VAR_NAME
from tests.fixtures import BaseTestCase

from .server_stubs import colab_stub as stub
from .server_stubs import serv

VERSIONED_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/13b/1"
LATEST_MODEL_VERSION = 2
UNVERSIONED_MODEL_HANDLE = "metaresearch/llama-2/pyTorch/13b"
TEST_FILEPATH = "config.json"
UNAVAILABLE_MODEL_HANDLE = "unavailable/model/handle/colab/1"


class TestColabCacheModelDownload(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        # Important, to match Colab TBE_RUNTIME_ADDR value in production, we don't prepend `http://`.
        # The `http://` is prepended inside the ColabClient class.
        cls.server = serv.start_server(stub.app, TBE_RUNTIME_ADDR_ENV_VAR_NAME, "localhost:7779")

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_unversioned_model_download(self) -> None:
        with stub.create_env():
            model_path = kagglehub.model_download(UNVERSIONED_MODEL_HANDLE)
            self.assertTrue(model_path.endswith("/2"))
            self.assertEqual(["config.json", "model.keras"], sorted(os.listdir(model_path)))

    def test_versioned_model_download(self) -> None:
        with stub.create_env():
            model_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE)
            self.assertTrue(model_path.endswith("/1"))
            self.assertEqual(["config.json"], sorted(os.listdir(model_path)))

    def test_versioned_model_download_with_path(self) -> None:
        with stub.create_env():
            model_file_path = kagglehub.model_download(VERSIONED_MODEL_HANDLE, "config.json")
            self.assertTrue(model_file_path.endswith("config.json"))
            self.assertTrue(os.path.isfile(model_file_path))

    def test_unversioned_model_download_with_path(self) -> None:
        with stub.create_env():
            model_file_path = kagglehub.model_download(UNVERSIONED_MODEL_HANDLE, "config.json")
            self.assertTrue(model_file_path.endswith("config.json"))
            self.assertTrue(os.path.isfile(model_file_path))

    def test_versioned_model_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.model_download(VERSIONED_MODEL_HANDLE, "missing.txt")

    def test_unversioned_model_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.model_download(UNVERSIONED_MODEL_HANDLE, "missing.txt")

    def test_colab_resolver_skipped_when_disable_colab_cache_env_var_name(self) -> None:
        with mock.patch.dict(os.environ, {DISABLE_COLAB_CACHE_ENV_VAR_NAME: "true"}):
            with stub.create_env():
                # Assert that a ConnectionError is set (uses HTTP server which is not set)
                with self.assertRaises(requests.exceptions.ConnectionError):
                    kagglehub.model_download(VERSIONED_MODEL_HANDLE)

    def test_versioned_model_download_bad_handle_raises(self) -> None:
        with self.assertRaises(ValueError):
            kagglehub.model_download("bad handle")


class TestNoInternetColabCacheModelDownload(BaseTestCase):
    def test_colab_resolver_skipped_when_model_not_present(self) -> None:
        with stub.create_env():
            # Assert that a ConnectionError is set (uses HTTP server which is not set)
            with self.assertRaises(requests.exceptions.ConnectionError):
                kagglehub.model_download(UNAVAILABLE_MODEL_HANDLE)



import hashlib
import os

from flask import Flask, Response, jsonify
from flask.typing import ResponseReturnValue

from kagglehub.integrity import to_b64_digest
from tests.utils import AUTO_COMPRESSED_FILE_NAME, add_mock_gcs_route, get_gcs_redirect_response, get_test_file_path

app = Flask(__name__)
add_mock_gcs_route(app)

TARGZ_ARCHIVE_HANDLE = "competition-targz"

# See https://cloud.google.com/storage/docs/xml-api/reference-headers#xgooghash
GCS_HASH_HEADER = "x-goog-hash"
LAST_MODIFIED = "Last-Modified"
LAST_MODIFIED_DATE = "Thu, 02 Mar 2020 02:17:12 GMT"


@app.route("/", methods=["HEAD"])
def head() -> ResponseReturnValue:
    return "", 200


@app.route("/api/v1/competitions/data/download-all/<competition_slug>", methods=["GET"])
def competition_download(competition_slug: str) -> ResponseReturnValue:
    handle = f"{competition_slug}"

    test_file_path = get_test_file_path("foo.txt.zip")
    content_type = "application/zip"

    if handle in TARGZ_ARCHIVE_HANDLE:
        test_file_path = get_test_file_path("archive.tar.gz")
        content_type = "application/x-gzip"

    with open(test_file_path, "rb") as f:
        content = f.read()
        file_hash = hashlib.md5()
        file_hash.update(content)
        resp = Response()
        resp.headers[GCS_HASH_HEADER] = f"md5={to_b64_digest(file_hash)}"
        resp.headers[LAST_MODIFIED] = LAST_MODIFIED_DATE
        resp.content_type = content_type
        resp.content_length = os.path.getsize(test_file_path)
        resp.data = content
        return resp, 200


@app.route("/api/v1/competitions/data/download/<competition_slug>/<file_name>", methods=["GET"])
def competition_download_file(competition_slug: str, file_name: str) -> ResponseReturnValue:
    _ = f"{competition_slug}"
    # This mimics behavior for our file downloads, where users request a file, but
    # receive a zipped version of the file from GCS.
    test_file = f"{file_name}.zip" if file_name is AUTO_COMPRESSED_FILE_NAME else file_name
    return get_gcs_redirect_response(test_file)


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"message": "Some response data", "error": str(e)}
    return jsonify(data), 404



import unittest

from requests import HTTPError

from kagglehub import model_download

from .utils import assert_files, create_test_cache, unauthenticated

HANDLE = "keras/bert/keras/bert_tiny_en_uncased/2"


class TestModelDownload(unittest.TestCase):
    def test_model_versioned_succeeds(self) -> None:
        with create_test_cache():
            actual_path = model_download(HANDLE)

            expected_files = [
                "assets/tokenizer/vocabulary.txt",
                "config.json",
                "metadata.json",
                "model.weights.h5",
                "tokenizer.json",
            ]
            assert_files(self, actual_path, expected_files)

    def test_model_unversioned_succeeds(self) -> None:
        with create_test_cache():
            unversioned_handle = "keras/bert/keras/bert_tiny_en_uncased"
            actual_path = model_download(unversioned_handle)

            expected_files = [
                "assets/tokenizer/vocabulary.txt",
                "config.json",
                "metadata.json",
                "model.weights.h5",
                "tokenizer.json",
            ]
            assert_files(self, actual_path, expected_files)

    def test_public_model_as_unauthenticated_succeeds(self) -> None:
        with create_test_cache():
            with unauthenticated():
                unversioned_handle = "keras/bert/keras/bert_tiny_en_uncased"
                actual_path = model_download(unversioned_handle)

                expected_files = [
                    "assets/tokenizer/vocabulary.txt",
                    "config.json",
                    "metadata.json",
                    "model.weights.h5",
                    "tokenizer.json",
                ]
                assert_files(self, actual_path, expected_files)

    def test_download_private_model_succeeds(self) -> None:
        with create_test_cache():
            actual_path = model_download("integrationtester/test-private-model/pyTorch/b0")

            expected_files = [
                "efficientnet-b0.pth",
            ]

            assert_files(self, actual_path, expected_files)

    def test_download_archive_model_many_files_succeeds(self) -> None:
        with create_test_cache():
            # If the model has > 25 files, we download the archive and uncompress it locally.
            actual_path = model_download("integrationtester/test-private-model/keras/many-files-with-subdirectories/1")

            expected_files = [f"assets/{i}.txt" for i in range(1, 26)] + [
                "config.json",
                "model.keras",
            ]
            assert_files(self, actual_path, expected_files)

    def test_download_multiple_files(self) -> None:
        with create_test_cache():
            file_paths = ["tokenizer.json", "config.json"]
            for p in file_paths:
                actual_path = model_download(HANDLE, path=p)
                assert_files(self, actual_path, [p])

    def test_download_with_incorrect_file_path(self) -> None:
        incorrect_path = "nonexistent/file/path"
        with self.assertRaises(HTTPError):
            model_download(HANDLE, path=incorrect_path)



import logging
from typing import Optional

from kagglehub import registry
from kagglehub.handle import parse_notebook_handle
from kagglehub.logger import EXTRA_CONSOLE_BLOCK

logger = logging.getLogger(__name__)


def notebook_output_download(handle: str, path: Optional[str] = None, *, force_download: Optional[bool] = False) -> str:
    """[WORK IN PROGRESS]

    Download notebook output files.

    Args:
        handle: (string) the notebook handle under https://kaggle.com/code.
        path: (string) Optional path to a file within the notebook output.
        force_download: (bool) Optional flag to force download motebook output, even if it's cached.


    Returns:
        A string representing the path to the requested notebook output files.
    """
    h = parse_notebook_handle(handle)
    logger.info(f"Downloading Notebook Output: {h.to_url()} ...", extra={**EXTRA_CONSOLE_BLOCK})
    return registry.notebook_output_resolver(h, path, force_download=force_download)



import abc
from typing import Generic, Optional, TypeVar

T = TypeVar("T")


class Resolver(Generic[T]):
    """Resolver base class: all resolvers inherit from this class."""

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def __call__(self, handle: T, path: Optional[str], *, force_download: Optional[bool] = False) -> str:
        """Resolves a handle into a path with the requested files.

        Args:
            handle: (string) the model handle to resolve.
            path: (string) Optional path to a file within the model bundle.
            force_download: (bool) Optional flag to force download a model, even if it's cached.

        Returns:
            A string representing the path
        """
        pass

    @abc.abstractmethod
    def is_supported(self, handle: T, path: Optional[str] = None) -> bool:
        """Returns whether the current environment supports this handle/path."""
        pass



import logging
from pathlib import Path
from typing import Optional, Union

from model_signing.api import SigningConfig

from kagglehub import registry
from kagglehub.gcs_upload import normalize_patterns, upload_files_and_directories
from kagglehub.handle import parse_model_handle
from kagglehub.logger import EXTRA_CONSOLE_BLOCK
from kagglehub.models_helpers import create_model_if_missing, create_model_instance_or_version, signing_token

logger = logging.getLogger(__name__)

# Patterns that are always ignored for model uploading.
DEFAULT_IGNORE_PATTERNS = [".git/", "*/.git/", ".cache/", ".huggingface/"]


def model_download(
    handle: str,
    path: Optional[str] = None,
    *,
    force_download: Optional[bool] = False,
) -> str:
    """Download model files.

    Args:
        handle: (string) the model handle.
        path: (string) Optional path to a file within the model bundle.
        force_download: (bool) Optional flag to force download a model, even if it's cached.

    Returns:
        A string representing the path to the requested model files.
    """
    h = parse_model_handle(handle)
    logger.info(f"Downloading Model: {h.to_url()} ...", extra={**EXTRA_CONSOLE_BLOCK})
    return registry.model_resolver(h, path, force_download=force_download)


def model_upload(
    handle: str,
    local_model_dir: str,
    license_name: Optional[str] = None,
    version_notes: str = "",
    ignore_patterns: Optional[Union[list[str], str]] = None,
    *,
    sigstore: Optional[bool] = False,
) -> None:
    """Upload model files.

    Args:
        handle: (str) the model handle.
        local_model_dir: (str) path to a file in a local directory.
        license_name: (str) model license.
        version_notes: (str, optional) model versions.
        ignore_patterns (str or list[str], optional):
            Additional ignore patterns to DEFAULT_IGNORE_PATTERNS.
            Files matching any of the patterns are not uploaded.
            Patterns are standard wildcards that can be matched by
            https://docs.python.org/3/library/fnmatch.html.
            Use a pattern ending with "/" to ignore the whole dir,
            e.g., ".git/" is equivalent to ".git/*".
        sigstore: (bool, optional)
            Creates a trasparent ledger on sigstore. User must be an admin/editor of the model.
    """
    # parse slug
    h = parse_model_handle(handle)
    logger.info(f"Uploading Model {h.to_url()} ...")
    if h.is_versioned():
        is_versioned_exception = "The model handle should not include the version"
        raise ValueError(is_versioned_exception)

    # Create the model if it doesn't already exist
    create_model_if_missing(h.owner, h.model)

    # Model can be non-existent. Get token after model creation so signing token can be authorized.
    try:
        if sigstore:
            token = signing_token(h.owner, h.model)
            if token:
                signing_file = Path(local_model_dir) / ".kaggle" / "signing.json"
                signing_file.parent.mkdir(exist_ok=True, parents=True)
                signing_file.unlink(missing_ok=True)
                # The below will throw an exception if the token can't be verified (Needs to be a production token)
                # Setting KAGGLE_API_ENDPOINT to localhost will throw the exception as stated above.
                SigningConfig().set_sigstore_signer(identity_token=token, use_staging=False).sign(
                    Path(local_model_dir), signing_file
                )
            else:
                # skips transparency log publishing as we are unable to get a token
                sigstore = False
                logger.warning("Unable to retrieve identity token. Skipping signing...")
    except Exception:
        logger.exception("Signing failed. Skipping signing...")
        sigstore = False

    # Upload the model files to GCS
    tokens = upload_files_and_directories(
        local_model_dir,
        item_type="model",
        ignore_patterns=normalize_patterns(default=DEFAULT_IGNORE_PATTERNS, additional=ignore_patterns),
    )

    create_model_instance_or_version(h, tokens, license_name, version_notes, sigstore=sigstore)



import io
import logging
from unittest import mock

import kagglehub
from kagglehub.auth import _capture_logger_output
from kagglehub.auth import _logger as logger
from kagglehub.config import get_kaggle_credentials
from kagglehub.exceptions import UnauthenticatedError
from tests.fixtures import BaseTestCase

from .server_stubs import auth_stub as stub
from .server_stubs import serv


class TestAuth(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_login_updates_global_credentials(self) -> None:
        login("lastplacelarry", "some-key")

        # Verify that the global variable contains the updated credentials
        credentials = get_kaggle_credentials()
        if credentials is None:
            self.fail("Credentials should not be None")
        self.assertEqual("lastplacelarry", credentials.username)
        self.assertEqual("some-key", credentials.key)

    def test_login_updates_global_credentials_no_validation(self) -> None:
        login("lastplacelarry", "some-key", validate_credentials=False)

        # Verify that the global variable contains the updated credentials
        credentials = get_kaggle_credentials()
        if credentials is None:
            self.fail("Credentials should not be None")
        self.assertEqual("lastplacelarry", credentials.username)
        self.assertEqual("some-key", credentials.key)

    def test_set_kaggle_credentials_raises_error_with_empty_username(self) -> None:
        with self.assertRaises(ValueError):
            login("", "some-key")

    def test_set_kaggle_credentials_raises_error_with_empty_api_key(self) -> None:
        with self.assertRaises(ValueError):
            login("lastplacelarry", "")

    def test_set_kaggle_credentials_raises_error_with_empty_username_api_key(self) -> None:
        with self.assertRaises(ValueError):
            login("", "")

    def test_login_returns_403_for_bad_credentials(self) -> None:
        output_stream = io.StringIO()
        handler = logging.StreamHandler(output_stream)
        logger.addHandler(handler)
        login("invalid", "invalid")

        captured_output = output_stream.getvalue()
        self.assertEqual(
            captured_output,
            "Invalid Kaggle credentials. You can check your credentials on the [Kaggle settings page](https://www.kaggle.com/settings/account).\n",
        )

    def test_capture_logger_output(self) -> None:
        with _capture_logger_output() as output:
            logger.info("This is an info message")
            logger.error("This is an error message")

        self.assertEqual(output.getvalue(), "This is an info message\nThis is an error message\n")

    def test_whoami_raises_unauthenticated_error(self) -> None:
        with self.assertRaises(UnauthenticatedError):
            kagglehub.whoami()

    def test_whoami_success(self) -> None:
        login("lastplacelarry", "some-key")

        result = kagglehub.whoami()
        self.assertEqual(result, {"username": "lastplacelarry"})


def login(username: str, api_key: str, validate_credentials: bool = True) -> None:  # noqa: FBT002, FBT001
    with mock.patch("builtins.input") as mock_input:
        with mock.patch("getpass.getpass") as mock_getpass:
            mock_input.side_effect = [username]
            mock_getpass.return_value = api_key
            kagglehub.login(validate_credentials=validate_credentials)



import sys
import unittest

from requests import HTTPError

from kagglehub import utility_script_install

from .utils import assert_files, create_test_cache


class TestUtilityScriptInstall(unittest.TestCase):
    def test_download_utility_script_succeeds(self) -> None:
        with create_test_cache():
            response_path = utility_script_install("integrationtester/utility-script")
            self.response_path = response_path

            expected_files = ["utility_script.py"]
            self.assertIn(response_path, sys.path)
            assert_files(self, response_path, expected_files)

    def test_download_private_utility_script_succeeds(self) -> None:
        with create_test_cache():
            response_path = utility_script_install("integrationtester/private-utility-script")
            self.response_path = response_path

            self.assertIn(response_path, sys.path)
            expected_files = ["private_utility_script.py"]
            assert_files(self, response_path, expected_files)

    def test_download_non_utility_script_sys_path_not_updated(self) -> None:
        with create_test_cache():
            response_path = utility_script_install("alexisbcook/titanic-tutorial")
            self.response_path = response_path

            self.assertNotIn(response_path, sys.path)
            expected_files = ["submission.csv"]
            assert_files(self, response_path, expected_files)

    def test_download_non_existent_utility_script_fails(self) -> None:
        with create_test_cache(), self.assertRaises(HTTPError):
            utility_script_install("integrationtester/i-dont-exist")

    def tearDown(self) -> None:
        if hasattr(self, "response_path") and self.response_path in sys.path:
            sys.path.remove(self.response_path)



import os
from datetime import datetime, timezone
from typing import Optional

import requests

import kagglehub
from kagglehub.cache import COMPETITIONS_CACHE_SUBFOLDER, get_cached_archive_path
from kagglehub.handle import parse_competition_handle
from tests.fixtures import BaseTestCase

from .server_stubs import competition_download_stub as stub
from .server_stubs import serv
from .utils import AUTO_COMPRESSED_FILE_NAME, create_test_cache

INVALID_ARCHIVE_COMPETITION_HANDLE = "invalid/invalid"
COMPETITION_HANDLE = "titanic"
TEST_FILEPATH = "foo.txt"
TEST_CONTENTS = "foo"
AUTO_COMPRESSED_CONTENTS = """"shape","degrees","sides","color","date"
"square",360,4,"blue","2024-12-17"
"circle",360,,"red","2023-08-01"
"triangle",180,3,"green","2022-01-05"
"""

EXPECTED_COMPETITION_SUBDIR = os.path.join(COMPETITIONS_CACHE_SUBFOLDER, "titanic")
EXPECTED_COMPETITION_SUBPATH = os.path.join(
    COMPETITIONS_CACHE_SUBFOLDER,
    "titanic",
)


class TestHttpCompetitionDownload(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)
        os.environ["KAGGLE_USERNAME"] = "fakeUser"
        os.environ["KAGGLE_KEY"] = "fakeKaggleKey"

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def _download_competition_and_assert_downloaded(
        self,
        d: str,
        competition_handle: str,
        expected_subdir_or_subpath: str,
        expected_files: Optional[list[str]] = None,
        **kwargs,  # noqa: ANN003
    ) -> None:
        if not expected_files:
            expected_files = ["foo.txt"]

        # Download the full competitions and ensure all files are there.
        competition_path = kagglehub.competition_download(competition_handle, **kwargs)
        archive_path = get_cached_archive_path(parse_competition_handle(competition_handle))

        self.assertEqual(os.path.join(d, expected_subdir_or_subpath), competition_path)
        self.assertEqual(sorted(expected_files), sorted(os.listdir(competition_path)))
        # Assert that the archive file has been deleted
        self.assertFalse(os.path.exists(archive_path))

    def _download_test_file_and_assert_downloaded(
        self,
        d: str,
        competition_handle: str,
        **kwargs,  # noqa: ANN003
    ) -> None:
        competition_path = kagglehub.competition_download(competition_handle, path=TEST_FILEPATH, **kwargs)

        self.assertEqual(os.path.join(d, EXPECTED_COMPETITION_SUBPATH, TEST_FILEPATH), competition_path)
        with open(competition_path) as competition_file:
            self.assertEqual(TEST_CONTENTS, competition_file.readline())

    def _download_test_file_and_assert_downloaded_auto_compressed(
        self,
        d: str,
        competition_handle: str,
        **kwargs,  # noqa: ANN003
    ) -> None:
        competition_path = kagglehub.competition_download(competition_handle, path=AUTO_COMPRESSED_FILE_NAME, **kwargs)
        self.assertEqual(os.path.join(d, EXPECTED_COMPETITION_SUBPATH, AUTO_COMPRESSED_FILE_NAME), competition_path)
        with open(competition_path) as competition_file:
            self.assertEqual(AUTO_COMPRESSED_CONTENTS, competition_file.read())

    def test_competition_download(self) -> None:
        with create_test_cache() as d:
            self._download_competition_and_assert_downloaded(d, COMPETITION_HANDLE, EXPECTED_COMPETITION_SUBDIR)

    def test_competition_targz_archive_download(self) -> None:
        with create_test_cache() as d:
            self._download_competition_and_assert_downloaded(
                d,
                stub.TARGZ_ARCHIVE_HANDLE,
                f"{COMPETITIONS_CACHE_SUBFOLDER}/{stub.TARGZ_ARCHIVE_HANDLE}",
                expected_files=[f"{i}.txt" for i in range(1, 51)],
            )

    def test_competition_download_bad_archive(self) -> None:
        with create_test_cache():
            with self.assertRaises(ValueError):
                kagglehub.competition_download(INVALID_ARCHIVE_COMPETITION_HANDLE)

    def test_competition_full_download_with_file_already_cached(self) -> None:
        with create_test_cache() as d:
            # Download a single file first
            kagglehub.competition_download(COMPETITION_HANDLE, path=TEST_FILEPATH)

            self._download_competition_and_assert_downloaded(d, COMPETITION_HANDLE, EXPECTED_COMPETITION_SUBDIR)

    def test_competition_download_with_force_download(self) -> None:
        with create_test_cache() as d:
            self._download_competition_and_assert_downloaded(
                d, COMPETITION_HANDLE, EXPECTED_COMPETITION_SUBDIR, force_download=True
            )

    def test_competition_download_ignored_cache_when_lastest_is_newer(self) -> None:
        with create_test_cache() as d:
            path = kagglehub.competition_download(COMPETITION_HANDLE)
            # force cached file to be out of date. We set it back to March 02 2000.
            test_date = 951955200
            os.utime(os.path.join(d, EXPECTED_COMPETITION_SUBDIR), (test_date, test_date))
            old_date = datetime.fromtimestamp(os.path.getmtime(path), tz=timezone.utc)

            # Latest version is from March 02 2020.
            path = kagglehub.competition_download(COMPETITION_HANDLE)

            # New cache file is current day.
            new_date = datetime.fromtimestamp(os.path.getmtime(path), tz=timezone.utc)
            self.assertEqual(os.path.join(d, EXPECTED_COMPETITION_SUBDIR), path)
            self.assertGreater(new_date, old_date)

    def test_competition_download_with_path(self) -> None:
        with create_test_cache() as d:
            self._download_test_file_and_assert_downloaded(d, COMPETITION_HANDLE)

    def test_competition_download_with_path_auto_compressed(self) -> None:
        with create_test_cache() as d:
            self._download_test_file_and_assert_downloaded_auto_compressed(d, COMPETITION_HANDLE)


class TestHttpNoInternet(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        os.environ["KAGGLE_USERNAME"] = "fakeUser"
        os.environ["KAGGLE_KEY"] = "fakeKaggleKey"

    def test_competition_download_already_cached_with_no_internet(self) -> None:
        with create_test_cache() as d:
            server = serv.start_server(stub.app)
            kagglehub.competition_download(COMPETITION_HANDLE)
            server.shutdown()

            path = kagglehub.competition_download(COMPETITION_HANDLE)

            self.assertEqual(os.path.join(d, EXPECTED_COMPETITION_SUBDIR), path)

    def test_competition_download_path_already_cached_with_no_internet(self) -> None:
        with create_test_cache() as d:
            server = serv.start_server(stub.app)
            path = kagglehub.competition_download(COMPETITION_HANDLE, path=TEST_FILEPATH)
            server.shutdown()

            path = kagglehub.competition_download(COMPETITION_HANDLE, path=TEST_FILEPATH)

            self.assertEqual(os.path.join(d, EXPECTED_COMPETITION_SUBPATH, TEST_FILEPATH), path)

    def test_competition_download_already_cached_with_force_download_no_internet(self) -> None:
        with create_test_cache():
            server = serv.start_server(stub.app)
            kagglehub.competition_download(COMPETITION_HANDLE)
            server.shutdown()

            # No internet should throw an error.
            with self.assertRaises(requests.exceptions.ConnectionError):
                kagglehub.competition_download(COMPETITION_HANDLE, force_download=True)

    def test_competition_download_with_path_already_cached_with_force_download_no_internet(self) -> None:
        with create_test_cache():
            server = serv.start_server(stub.app)
            kagglehub.competition_download(COMPETITION_HANDLE, path=TEST_FILEPATH)
            server.shutdown()

            # No internet should throw an error.
            with self.assertRaises(requests.exceptions.ConnectionError):
                kagglehub.competition_download(COMPETITION_HANDLE, path=TEST_FILEPATH, force_download=True)



import kagglehub
from kagglehub import KaggleDatasetAdapter

dataset = kagglehub.load_dataset(
    KaggleDatasetAdapter.PANDAS,
    "wyattowalsh/basketball",
    "nba.sqlite",
    sql_query="SELECT person_id, player_name FROM draft_history",
)
print("Records in dataset: ", dataset.shape)
print(dataset.head())



import logging
import os
import sys
from logging import LogRecord
from logging.handlers import RotatingFileHandler
from pathlib import Path
from typing import Callable, Optional

from kagglehub.config import get_log_verbosity

KAGGLE_LOGGING_ROOT_DIR_ENV_VAR_NAME = "KAGGLE_LOGGING_ROOT_DIR"
KAGGLE_LOGGING_ENABLED_ENV_VAR_NAME = "KAGGLE_LOGGING_ENABLED"

_FILE_BLOCK_KEY = "kaggle_file"
EXTRA_FILE_BLOCK = {"block": _FILE_BLOCK_KEY}
_CONSOLE_BLOCK_KEY = "console"
EXTRA_CONSOLE_BLOCK = {"block": _CONSOLE_BLOCK_KEY}


def _block_logrecord_factory(elements: list[str]) -> Callable[[LogRecord], bool]:
    """Filter to block log statements based on data attributes
    Args:
        elements: The value for the key 'block'. For example log.info("..", extra={"block" : "console"})
    """

    def _filter(record: LogRecord) -> bool:
        if hasattr(record, "block"):
            if record.block in elements:
                return False
        return True

    return _filter


def _configure_logger(log_dir: Optional[Path] = None) -> None:
    library_name = __name__.split(".")[0]  # i.e. "kagglehub"
    library_logger = logging.getLogger(library_name)
    while library_logger.handlers:
        handler = library_logger.handlers.pop()
        while handler.filters:
            handler.filters.pop()
    logging_enabled = os.environ.get(KAGGLE_LOGGING_ENABLED_ENV_VAR_NAME, False)
    if str(logging_enabled).lower() in ("true", 1):
        log_root_dir = Path(os.environ.get(KAGGLE_LOGGING_ROOT_DIR_ENV_VAR_NAME, Path.home()))
        log_dir = log_root_dir / ".kaggle" / "logs" if log_dir is None else log_dir
        log_dir.mkdir(exist_ok=True, parents=True)
        file_handler = RotatingFileHandler(
            str(log_dir / "kagglehub.log"), maxBytes=1024 * 1024 * 5, backupCount=5, delay=True
        )
        file_handler.addFilter(_block_logrecord_factory([_FILE_BLOCK_KEY]))
        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(threadName)s - %(funcName)s - %(message)s"
        )
        file_handler.setFormatter(formatter)
        file_handler.setLevel(logging.DEBUG)
        library_logger.addHandler(file_handler)
    sh = logging.StreamHandler(sys.stdout)
    sh.addFilter(_block_logrecord_factory([_CONSOLE_BLOCK_KEY]))
    sh.setLevel(get_log_verbosity())
    library_logger.addHandler(sh)
    # Disable propagation of the library log outputs.
    # This prevents the same message again from being printed again if a root logger is defined.
    library_logger.propagate = False
    library_logger.setLevel(get_log_verbosity())


_configure_logger()



import os
from typing import Any
from unittest.mock import MagicMock, patch

from requests import Response

from kagglehub.datasets import KaggleDatasetAdapter, load_dataset
from kagglehub.exceptions import KaggleApiHTTPError
from tests.fixtures import BaseTestCase

from .server_stubs import dataset_download_stub as stub
from .server_stubs import serv
from .utils import AUTO_COMPRESSED_FILE_NAME, create_test_cache

DATASET_HANDLE = "lastplacelarry/fake-dataset"
TEST_SPLIT_SIZE = 1
TRAIN_SPLIT_SIZE = 2
INVALID_KWARG = "this_is_not_a_kwarg"
TEXT_FILE = "foo.txt"
EXCEL_FILE = "my-spreadsheet.xlsx"
SHAPES_COLUMNS = ["shape", "degrees", "sides", "color", "date"]
SHAPES_ROW_COUNT = 3


class TestLoadHfDataset(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def _load_hf_dataset_with_invalid_file_type_and_assert_raises(self) -> None:
        with self.assertRaises(ValueError) as cm:
            load_dataset(
                KaggleDatasetAdapter.HUGGING_FACE,
                DATASET_HANDLE,
                TEXT_FILE,
            )
        self.assertIn(f"Unsupported file extension: '{os.path.splitext(TEXT_FILE)[1]}'", str(cm.exception))

    def _load_hf_dataset_with_multiple_tables_and_assert_raises(self) -> None:
        with self.assertRaises(ValueError) as cm:
            load_dataset(
                KaggleDatasetAdapter.HUGGING_FACE, DATASET_HANDLE, EXCEL_FILE, pandas_kwargs={"sheet_name": None}
            )
        self.assertIn(
            "Loading a Huggingface dataset requires the production of exactly one DataFrame", str(cm.exception)
        )

    def _load_hf_dataset_and_assert_loaded(self) -> None:
        hf_dataset = load_dataset(KaggleDatasetAdapter.HUGGING_FACE, DATASET_HANDLE, AUTO_COMPRESSED_FILE_NAME)
        self.assertEqual(SHAPES_ROW_COUNT, hf_dataset.num_rows)
        self.assertEqual(SHAPES_COLUMNS, hf_dataset.column_names)

    def _load_hf_dataset_with_valid_kwargs_and_assert_loaded(self) -> None:
        hf_dataset = load_dataset(
            KaggleDatasetAdapter.HUGGING_FACE,
            DATASET_HANDLE,
            AUTO_COMPRESSED_FILE_NAME,
            hf_kwargs={"preserve_index": True},
        )
        self.assertEqual(SHAPES_ROW_COUNT, hf_dataset.num_rows)
        self.assertEqual([*SHAPES_COLUMNS, "__index_level_0__"], hf_dataset.column_names)

    def _load_hf_dataset_with_invalid_kwargs_and_assert_raises(self) -> None:
        with self.assertRaises(TypeError) as cm:
            load_dataset(
                KaggleDatasetAdapter.HUGGING_FACE,
                DATASET_HANDLE,
                AUTO_COMPRESSED_FILE_NAME,
                hf_kwargs={INVALID_KWARG: 777},
            )
        self.assertIn(INVALID_KWARG, str(cm.exception))

    def _load_hf_dataset_with_splits_and_assert_loaded(self) -> None:
        hf_dataset = load_dataset(KaggleDatasetAdapter.HUGGING_FACE, DATASET_HANDLE, AUTO_COMPRESSED_FILE_NAME)
        dataset_splits = hf_dataset.train_test_split(
            test_size=TEST_SPLIT_SIZE, train_size=TRAIN_SPLIT_SIZE, shuffle=False
        )
        for split_name, dataset in dataset_splits.items():
            self.assertEqual(TEST_SPLIT_SIZE if split_name == "test" else TRAIN_SPLIT_SIZE, dataset.num_rows)
            self.assertEqual(SHAPES_COLUMNS, dataset.column_names)

    def test_hf_dataset_with_invalid_file_type_raises(self) -> None:
        with create_test_cache():
            self._load_hf_dataset_with_invalid_file_type_and_assert_raises()

    def test_hf_dataset_with_multiple_tables_raises(self) -> None:
        with create_test_cache():
            self._load_hf_dataset_with_multiple_tables_and_assert_raises()

    def test_hf_dataset_succeeds(self) -> None:
        with create_test_cache():
            self._load_hf_dataset_and_assert_loaded()

    def test_hf_dataset_with_valid_kwargs_succeeds(self) -> None:
        with create_test_cache():
            self._load_hf_dataset_with_valid_kwargs_and_assert_loaded()

    def test_hf_dataset_with_invalid_kwargs_raises(self) -> None:
        with create_test_cache():
            self._load_hf_dataset_with_invalid_kwargs_and_assert_raises()

    def test_hf_dataset_with_splits_succeeds(self) -> None:
        with create_test_cache():
            self._load_hf_dataset_with_splits_and_assert_loaded()

    @patch("requests.get")
    def test_hf_dataset_sends_user_agent(self, mock_get: MagicMock) -> None:
        # Just mock a bad response since all we care to check is that the request headers are right
        response = Response()
        response.status_code = 400
        mock_get.return_value = response

        with self.assertRaises(KaggleApiHTTPError):
            load_dataset(KaggleDatasetAdapter.HUGGING_FACE, DATASET_HANDLE, AUTO_COMPRESSED_FILE_NAME)
        self.assertIn("hugging_face_data_loader", mock_get.call_args.kwargs["headers"]["User-Agent"])


class TestLoadPandasDataset(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def _load_pandas_dataset_with_invalid_file_type_and_assert_raises(self) -> None:
        with self.assertRaises(ValueError) as cm:
            load_dataset(
                KaggleDatasetAdapter.PANDAS,
                DATASET_HANDLE,
                TEXT_FILE,
            )
        self.assertIn(f"Unsupported file extension: '{os.path.splitext(TEXT_FILE)[1]}'", str(cm.exception))

    def _load_pandas_simple_dataset_and_assert_loaded(
        self,
        file_extension: str,
        pandas_kwargs: Any = None,  # noqa: ANN401
    ) -> None:
        df = load_dataset(
            KaggleDatasetAdapter.PANDAS, DATASET_HANDLE, f"shapes.{file_extension}", pandas_kwargs=pandas_kwargs
        )
        self.assertEqual(SHAPES_ROW_COUNT, len(df))
        self.assertEqual(SHAPES_COLUMNS, list(df.columns))

    def _load_pandas_sqlite_dataset_and_assert_loaded(self) -> None:
        df = load_dataset(KaggleDatasetAdapter.PANDAS, DATASET_HANDLE, "shapes.db", sql_query="SELECT * FROM shapes")
        self.assertEqual(SHAPES_ROW_COUNT, len(df))
        self.assertEqual(SHAPES_COLUMNS, list(df.columns))

    def _load_pandas_dataset_with_multiple_tables_and_assert_loaded(self) -> None:
        result = load_dataset(
            KaggleDatasetAdapter.PANDAS, DATASET_HANDLE, EXCEL_FILE, pandas_kwargs={"sheet_name": None}
        )
        self.assertTrue(result, isinstance(result, dict))
        self.assertEqual(["Cars", "Animals"], list(result.keys()))

    def _load_pandas_dataset_with_valid_kwargs_and_assert_loaded(self) -> None:
        expected_columns = ["degrees"]
        df = load_dataset(
            KaggleDatasetAdapter.PANDAS,
            DATASET_HANDLE,
            AUTO_COMPRESSED_FILE_NAME,
            pandas_kwargs={"usecols": expected_columns},
        )
        self.assertEqual(SHAPES_ROW_COUNT, len(df))
        self.assertEqual(expected_columns, list(df.columns))

    def _load_pandas_dataset_with_invalid_kwargs_and_assert_raises(self) -> None:
        with self.assertRaises(ValueError) as cm:
            load_dataset(
                KaggleDatasetAdapter.PANDAS,
                DATASET_HANDLE,
                AUTO_COMPRESSED_FILE_NAME,
                pandas_kwargs={INVALID_KWARG: 777},
            )
        self.assertIn(INVALID_KWARG, str(cm.exception))

    def test_pandas_dataset_with_invalid_file_type_raises(self) -> None:
        with create_test_cache():
            self._load_pandas_dataset_with_invalid_file_type_and_assert_raises()

    def test_pandas_dataset_with_multiple_tables_succeeds(self) -> None:
        with create_test_cache():
            self._load_pandas_dataset_with_multiple_tables_and_assert_loaded()

    def test_pandas_dataset_with_valid_kwargs_succeeds(self) -> None:
        with create_test_cache():
            self._load_pandas_dataset_with_valid_kwargs_and_assert_loaded()

    def test_pandas_dataset_with_invalid_kwargs_raises(self) -> None:
        with create_test_cache():
            self._load_pandas_dataset_with_invalid_kwargs_and_assert_raises()

    def test_pandas_simple_dataset_succeeds(self) -> None:
        # This would be better as a parameterized test, but that doesn't work for subclasses:
        # https://docs.pytest.org/en/stable/how-to/unittest.html#pytest-features-in-unittest-testcase-subclasses
        test_cases = [
            ("csv", {}),
            ("tsv", {}),
            ("xml", {"parser": "etree"}),  # The default parser requires a pip install
            ("parquet", {}),
            ("feather", {}),
            ("json", {}),
            ("jsonl", {}),
        ]
        for test_case in test_cases:
            with create_test_cache():
                self._load_pandas_simple_dataset_and_assert_loaded(test_case[0], test_case[1])

    def test_pandas_sqlite_dataset_succeeds(self) -> None:
        with create_test_cache():
            self._load_pandas_sqlite_dataset_and_assert_loaded()

    @patch("requests.get")
    def test_pandas_dataset_sends_user_agent(self, mock_get: MagicMock) -> None:
        # Just mock a bad response since all we care to check is that the request headers are right
        response = Response()
        response.status_code = 400
        mock_get.return_value = response

        with self.assertRaises(KaggleApiHTTPError):
            load_dataset(KaggleDatasetAdapter.PANDAS, DATASET_HANDLE, AUTO_COMPRESSED_FILE_NAME)
        self.assertIn("pandas_data_loader", mock_get.call_args.kwargs["headers"]["User-Agent"])



import kagglehub
from kagglehub import KaggleDatasetAdapter

dataset = kagglehub.load_dataset(
    KaggleDatasetAdapter.HUGGING_FACE,
    "unsdsn/world-happiness",
    "2016.csv",
)
print("Records in dataset: ", dataset.num_rows)
print(dataset)



import os
from unittest import mock

import requests

import kagglehub
from kagglehub.config import DISABLE_COLAB_CACHE_ENV_VAR_NAME, TBE_RUNTIME_ADDR_ENV_VAR_NAME
from tests.fixtures import BaseTestCase

from .server_stubs import colab_stub as stub
from .server_stubs import serv

VERSIONED_DATASET_HANDLE = "sarahjeffreson/featured-spotify-artiststracks-with-metadata/versions/1"
UNVERSIONED_DATASET_HANDLE = "sarahjeffreson/featured-spotify-artiststracks-with-metadata"
TEST_FILEPATH = "foo.txt"
TEST_CONTENTS = "foo\n"
UNAVAILABLE_DATASET_HANDLE = "unavailable/dataset/versions/1"


class TestColabCacheDatasetDownload(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        # Important, to match Colab TBE_RUNTIME_ADDR value in production, we don't prepend `http://`.
        # The `http://` is prepended inside the ColabClient class.
        cls.server = serv.start_server(stub.app, TBE_RUNTIME_ADDR_ENV_VAR_NAME, "localhost:7779")

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_unversioned_dataset_download(self) -> None:
        with stub.create_env():
            dataset_path = kagglehub.dataset_download(UNVERSIONED_DATASET_HANDLE)
            self.assertTrue(dataset_path.endswith("/2"))
            self.assertEqual(["bar.csv", "foo.txt"], sorted(os.listdir(dataset_path)))

    def test_versioned_dataset_download(self) -> None:
        with stub.create_env():
            dataset_path = kagglehub.dataset_download(VERSIONED_DATASET_HANDLE)
            self.assertTrue(dataset_path.endswith("/1"))
            self.assertEqual(["foo.txt"], sorted(os.listdir(dataset_path)))

    def test_versioned_dataset_download_with_path(self) -> None:
        with stub.create_env():
            dataset_file_path = kagglehub.dataset_download(VERSIONED_DATASET_HANDLE, "foo.txt")
            self.assertTrue(dataset_file_path.endswith("foo.txt"))
            self.assertTrue(os.path.isfile(dataset_file_path))

    def test_unversioned_dataset_download_with_path(self) -> None:
        with stub.create_env():
            dataset_file_path = kagglehub.dataset_download(UNVERSIONED_DATASET_HANDLE, "bar.csv")
            self.assertTrue(dataset_file_path.endswith("bar.csv"))
            self.assertTrue(os.path.isfile(dataset_file_path))

    def test_versioned_dataset_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.dataset_download(VERSIONED_DATASET_HANDLE, "missing.txt")

    def test_unversioned_dataset_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.dataset_download(UNVERSIONED_DATASET_HANDLE, "missing.txt")

    def test_colab_resolver_skipped_when_disable_colab_cache_env_var_name(self) -> None:
        with mock.patch.dict(os.environ, {DISABLE_COLAB_CACHE_ENV_VAR_NAME: "true"}):
            with stub.create_env():
                # Assert that a ConnectionError is set (uses HTTP server which is not set)
                with self.assertRaises(requests.exceptions.ConnectionError):
                    kagglehub.dataset_download(VERSIONED_DATASET_HANDLE)

    def test_versioned_dataset_download_bad_handle_raises(self) -> None:
        with self.assertRaises(ValueError):
            kagglehub.dataset_download("bad handle")


class TestNoInternetColabCacheModelDownload(BaseTestCase):
    def test_colab_resolver_skipped_when_dataset_not_present(self) -> None:
        with stub.create_env():
            # Assert that a ConnectionError is set (uses HTTP server which is not set)
            with self.assertRaises(requests.exceptions.ConnectionError):
                kagglehub.dataset_download(UNAVAILABLE_DATASET_HANDLE)



import unittest

from requests import HTTPError

from kagglehub import notebook_output_download

from .utils import assert_files, create_test_cache, unauthenticated

VERSIONED_NOTEBOOK_HANDLE = "alexisbcook/titanic-tutorial/versions/1"
UNVERSIONED_NOTEBOOK_HANDLE = "alexisbcook/titanic-tutorial"


class TestModelDownload(unittest.TestCase):
    def test_download_notebook_unversioned_output_succeeds(self) -> None:
        with create_test_cache():
            actual_path = notebook_output_download(UNVERSIONED_NOTEBOOK_HANDLE)

            expected_files = ["submission.csv"]
            assert_files(self, actual_path, expected_files)

    def test_download_notebook_versioned_output_succeeds(self) -> None:
        with create_test_cache():
            actual_path = notebook_output_download(VERSIONED_NOTEBOOK_HANDLE)

            expected_files = ["my_submission.csv"]
            assert_files(self, actual_path, expected_files)

    def test_download_both_notebook_outputs_succeeds(self) -> None:
        with create_test_cache():
            versioned_path = notebook_output_download(VERSIONED_NOTEBOOK_HANDLE)
            unversioned_path = notebook_output_download(UNVERSIONED_NOTEBOOK_HANDLE)

            expected_versioned_files = ["my_submission.csv"]
            expected_unversioned_files = ["submission.csv"]
            assert_files(self, versioned_path, expected_versioned_files)
            assert_files(self, unversioned_path, expected_unversioned_files)

    def test_download_public_notebook_output_as_unauthenticated_succeeds(self) -> None:
        with create_test_cache():
            with unauthenticated():
                actual_path = notebook_output_download(UNVERSIONED_NOTEBOOK_HANDLE)

                expected_files = ["submission.csv"]
                assert_files(self, actual_path, expected_files)

    def test_download_private_notebook_output_succeeds(self) -> None:
        with create_test_cache():
            actual_path = notebook_output_download("integrationtester/private-titanic-tutorial")

            expected_files = ["submission-01.csv", "submission-02.csv"]

            assert_files(self, actual_path, expected_files)

    def test_download_private_notebook_output_single_file_succeeds(self) -> None:
        with create_test_cache():
            actual_path = notebook_output_download(
                "integrationtester/private-titanic-tutorial", path="submission-02.csv"
            )

            expected_files = ["submission-02.csv"]

            assert_files(self, actual_path, expected_files)

    def test_download_large_notebook_output_warns(self) -> None:
        handle = "integrationtester/titanic-tutorial-many-output-files"
        with create_test_cache():
            # If the model has > 25 files, we warn the user that it's not supported yet
            # TODO(b/379761520): add support for .tar.gz archived downloads
            notebook_output_download(handle)
            msg = f"Too many files in {handle} (capped at 25). Unable to download notebook output."
            self.assertLogs(msg, "WARNING")

    def test_download_private_notebook_output_with_incorrect_file_path_fails(self) -> None:
        with create_test_cache(), self.assertRaises(HTTPError):
            notebook_output_download("integrationtester/titanic-tutorial", path="submission-03.csv")



import datetime
import threading
from dataclasses import dataclass, field
from datetime import timedelta

import jwt
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from flask import Flask, jsonify, request
from flask.typing import ResponseReturnValue

from ..utils import resolve_endpoint

app = Flask(__name__)

APACHE_LICENSE = "Apache 2.0"
ALLOWED_LICENSE_VALUES = APACHE_LICENSE


@dataclass
class SharedData:
    files: list[str] = field(default_factory=list)
    simulate_308: bool = False
    blob_request_count: int = 0


shared_data: SharedData = SharedData()
lock = threading.Lock()


def _increment_blob_request() -> None:
    lock.acquire()
    shared_data.blob_request_count += 1
    lock.release()


def _add_file(file: str) -> None:
    lock.acquire()
    shared_data.files.append(file)
    lock.release()


def reset() -> None:
    lock.acquire()
    shared_data.files = []
    shared_data.blob_request_count = 0
    shared_data.simulate_308 = False
    lock.release()


def simulate_308(*, state: bool) -> None:
    lock.acquire()
    shared_data.simulate_308 = state
    lock.release()


@app.route("/", methods=["HEAD"])
def head() -> ResponseReturnValue:
    return "", 200


@app.route("/api/v1/models/<org_slug>/<model_slug>/get", methods=["GET"])
def model_get(org_slug: str, model_slug: str) -> ResponseReturnValue:
    data = {"message": f"Model exists {org_slug}/{model_slug} !"}
    return jsonify(data), 200


@app.route("/api/v1/models/<org_slug>/<model_slug>/<framework>/<variation>/get", methods=["GET"])
def model_get_instance(org_slug: str, model_slug: str, framework: str, variation: str) -> ResponseReturnValue:
    data = {"message": f"Instance exists {org_slug}/{model_slug}/{framework}/{variation} !"}
    return jsonify(data), 200


@app.route("/api/v1/models/<org_slug>/<model_slug>/<framework>/<variation>/<version>/get", methods=["GET"])
def model_get_instance_version(
    org_slug: str, model_slug: str, framework: str, variation: str, version: int
) -> ResponseReturnValue:
    data = {"message": f"Instance exists {org_slug}/{model_slug}/{framework}/{variation}/{version} !"}
    return jsonify(data), 200


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"message": "Some response data", "error": str(e)}
    return jsonify(data), 404


@app.route("/api/v1/models/create/new", methods=["POST"])
def model_create() -> ResponseReturnValue:
    data = {"status": "success", "message": "Model created successfully"}
    return jsonify(data), 200


@app.route("/api/v1/models/<org_slug>/<model_slug>/create/instance", methods=["POST"])
def model_instance_create_instance(org_slug: str, model_slug: str) -> ResponseReturnValue:
    post_data = request.get_json()
    if post_data.get("licenseName", "") not in ALLOWED_LICENSE_VALUES:
        data = {"error": f"bad: {request.path}"}
        return jsonify(data), 200
    data = {"status": "success", "message": f"Model Instance {org_slug}/{model_slug} created successfully"}
    return jsonify(data), 200


@app.route("/api/v1/models/<org_slug>/<model_slug>/<framework>/<variation>/create/version", methods=["POST"])
def model_instance_create_version(
    org_slug: str, model_slug: str, framework: str, variation: str
) -> ResponseReturnValue:
    post_data = request.get_json()
    if post_data.get("licenseName", "") not in ALLOWED_LICENSE_VALUES:
        data = {"error": f"bad: {request.path}"}
        return jsonify(data), 200
    data = {
        "status": "success",
        "message": f"Model Version {org_slug}/{model_slug}/{framework}/{variation} created successfully",
    }
    return jsonify(data), 200


@app.route("/api/v1/blobs/upload", methods=["POST"])
def blob_upload() -> ResponseReturnValue:
    if shared_data.simulate_308 and shared_data.blob_request_count < 0:
        _increment_blob_request()
        return "", 308
    address, port = resolve_endpoint()
    data = {
        "token": "dummy",
        "createUrl": f"http://{address}:{port}/upload/storage/v1/b/kaggle-models-data/o",
        "status": "success",
        "message": "Here is your token and Url",
    }
    post_data = request.get_json()
    _add_file(post_data["name"])
    _increment_blob_request()
    return jsonify(data), 200


@app.route("/upload/storage/v1/b/kaggle-models-data/o", methods=["PUT"])
def model_instance_create() -> ResponseReturnValue:
    data = {"status": "success", "message": "File uploaded"}
    return jsonify(data), 200


@app.route("/api/v1/models/signing/token", methods=["POST"])
def model_signing_token() -> ResponseReturnValue:
    now = datetime.datetime.now(tz=datetime.timezone.utc)
    # Generate the private key
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
    )
    # Serialize the private key to PEM format
    key = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption(),
    )
    token = jwt.encode(
        {
            "iss": "https://www.kaggle.com/api/v1/models/signing",
            "sub": "418540",
            "aud": "sigstore",
            "exp": int((now + timedelta(minutes=10)).timestamp()),
            "nbf": int(now.timestamp()),
            "iat": int(now.timestamp()),
            "email": "foo@gmail.com",
            "email_verified": True,
        },
        key,
        algorithm="RS256",
    )
    data = {"id_token": token}
    return jsonify(data), 200



import os
import sqlite3
from typing import Any, Callable, Optional, Union

# WARNING: This module is intended to be imported only at runtime, with
# specific error handling to inform users that they need to install the
# appropriate `pandas-datasets` extras in order to use these methods. Adding
# any new dependencies here must correspond with an addition to the
# list of `pandas-datasets` optional dependencies in pyproject.toml.
import pandas as pd

from kagglehub.datasets import dataset_download


# This is a thin wrapper around pd.read_sql_query so we get the connection
# closing for free after we've produced the DataFrame
def wrapped_read_sql_query(sql_query: str, path: str) -> pd.DataFrame:
    with sqlite3.connect(path) as conn:
        df = pd.read_sql_query(sql_query, conn)
        return df


# These are the currently supported read functions for the pandas adapter.
# Taken from https://pandas.pydata.org/docs/reference/io.html. More can be
# added as demand dictates.
SUPPORTED_READ_FUNCTIONS_BY_EXTENSION: dict[str, Callable] = {
    ".csv": pd.read_csv,
    ".tsv": pd.read_csv,  # Additional kwargs for the pandas method are defined below
    ".json": pd.read_json,
    ".jsonl": pd.read_json,  # Additional kwargs for the pandas method are defined below
    ".xml": pd.read_xml,
    ".parquet": pd.read_parquet,
    ".feather": pd.read_feather,
    # Technically, SQLite files can have any extension (or none), but these are the listed
    # ones on Wikipedia: https://en.wikipedia.org/wiki/SQLite. We can add more robust checking
    # by testing connections if users request it.
    ".sqlite": wrapped_read_sql_query,
    ".sqlite3": wrapped_read_sql_query,
    ".db": wrapped_read_sql_query,
    ".db3": wrapped_read_sql_query,
    ".s3db": wrapped_read_sql_query,
    ".dl3": wrapped_read_sql_query,
    # read_excel supports many file types:
    # https://pandas.pydata.org/docs/reference/api/pandas.read_excel.html
    ".xls": pd.read_excel,
    ".xlsx": pd.read_excel,
    ".xlsm": pd.read_excel,
    ".xlsb": pd.read_excel,
    ".odf": pd.read_excel,
    ".ods": pd.read_excel,
    ".odt": pd.read_excel,
}

# Certain extensions leverage a shared method but require additional static kwargs
STATIC_KWARGS_BY_EXTENSION: dict[str, dict[str, Union[str, bool]]] = {".tsv": {"sep": "\t"}, ".jsonl": {"lines": True}}
MISSING_SQL_QUERY_ERROR_MESSAGE = "Loading from a SQLite file requires a SQL query"


def load_pandas_dataset(
    handle: str,
    path: str,
    *,
    pandas_kwargs: Any = None,  # noqa: ANN401
    sql_query: Optional[str],
) -> Union[pd.DataFrame, dict[Union[int, str], pd.DataFrame]]:
    """Creates pandas DataFrame(s) from a file in the dataset

    Args:
        handle: (string) The dataset handle
        path: (string) Path to a file within the dataset
        pandas_kwargs:
            (dict) Optional set of kwargs to pass to the pandas `read_*` method while constructing the DataFrame(s)
        sql_query:
            (string) Argument to be used for SQLite files. Required when reading a SQLite file. See pandas documentation
            for details: https://pandas.pydata.org/docs/reference/api/pandas.read_sql_query.html

    Returns:
        - dict[int | str, DataFrame] for Excel-like files with multiple sheets
        - A pandas DataFrame for all others

    Raises:
        ValueError: If the file extension is not supported or the file fails to read
    """
    pandas_kwargs = {} if pandas_kwargs is None else pandas_kwargs
    file_extension = os.path.splitext(path)[1]
    read_function = _validate_read_function(file_extension, sql_query)

    # Now that everything has been validated, we can start downloading and processing
    filepath = dataset_download(handle, path)
    try:
        result = read_function(
            *_build_args(read_function, filepath, sql_query),
            **_build_kwargs(file_extension, pandas_kwargs),
        )
    except Exception as e:
        read_error_message = f"Error reading file: {e}"
        raise ValueError(read_error_message) from e

    return result


def _validate_read_function(file_extension: str, sql_query: Optional[str]) -> Callable:
    if file_extension not in SUPPORTED_READ_FUNCTIONS_BY_EXTENSION:
        extension_error_message = (
            f"Unsupported file extension: '{file_extension}'. "
            f"Supported file extensions are: {', '.join(SUPPORTED_READ_FUNCTIONS_BY_EXTENSION.keys())}"
        )
        raise ValueError(extension_error_message) from None

    read_function = SUPPORTED_READ_FUNCTIONS_BY_EXTENSION[file_extension]
    if read_function is wrapped_read_sql_query and not sql_query:
        raise ValueError(MISSING_SQL_QUERY_ERROR_MESSAGE)

    return read_function


def _build_args(read_function: Callable, path: str, sql_query: Optional[str]) -> list:
    # The presence of the sql_query arg was already validated in _validate_read_function
    return [path] if read_function != wrapped_read_sql_query else [sql_query, path]


def _build_kwargs(file_extension: str, pandas_kwargs: Any) -> dict:  # noqa: ANN401
    static_kwargs: dict[str, Any] = (
        {} if file_extension not in STATIC_KWARGS_BY_EXTENSION else STATIC_KWARGS_BY_EXTENSION[file_extension]
    )
    # NOTE: Order matters here as we're letting users override the static args. This may be valuable in the edge case
    # that a CSV/TSV has some other separator than what the extension indicates (we see this in real datasets).
    return {**static_kwargs, **pandas_kwargs}



import logging
import sys
from http import HTTPStatus
from typing import Optional

from kagglehub import registry
from kagglehub.clients import KaggleApiV1Client
from kagglehub.exceptions import KaggleApiHTTPError
from kagglehub.handle import UtilityScriptHandle, parse_utility_script_handle
from kagglehub.logger import EXTRA_CONSOLE_BLOCK

logger = logging.getLogger(__name__)


def utility_script_install(handle: str, *, force_download: Optional[bool] = False) -> str:
    """
    Downloads the utility script and adds the directory path to the system path.

    Args:
        handle: (string) the notebook handle under https://kaggle.com/code.
        force_download: (bool) Optional flag to force download motebook output, even if it's cached.


    Returns:
        A string representing the path to the requested notebook output files.
    """
    h = parse_utility_script_handle(handle)

    logger.info(f"Downloading Utility Script: {h.to_url()} ...", extra={**EXTRA_CONSOLE_BLOCK})
    utility_script_path = registry.notebook_output_resolver(h, path=None, force_download=force_download)

    if not _is_notebook_utility_script(h):
        logger.info(
            f"Notebook '{h.notebook}' by user '{h.owner}' is not a Utility Script"
            "\n and will not be added to system path"
        )
        return utility_script_path

    if utility_script_path not in sys.path:
        sys.path.append(utility_script_path)

    logger.info(f"Added {utility_script_path} to system path")
    return utility_script_path


def _is_notebook_utility_script(h: UtilityScriptHandle) -> bool:
    try:
        api_client = KaggleApiV1Client()
        json_response = api_client.get(f"kernels/pull/{h.owner}/{h.notebook}", h)

        category_ids = json_response["metadata"]["categoryIds"]
        return "utility script" in category_ids

    except KaggleApiHTTPError as e:
        if e.response is not None and e.response.status_code == HTTPStatus.NOT_FOUND:
            logger.info(f"Could not find '{h.owner}' metadata by user '{h.notebook}'.")
            return False
        else:
            raise (e)



import hashlib
import os
import re

from flask import Flask, Response, jsonify, make_response, request
from flask.typing import ResponseReturnValue

from kagglehub.integrity import to_b64_digest
from tests.utils import get_test_file_path

app = Flask(__name__)


@app.route("/", methods=["HEAD"])
def head() -> ResponseReturnValue:
    return "", 200


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"code": "404", "error": str(e), "message": "server side error"}
    return jsonify(data), 404


@app.route("/api/v1/content_type_mismatch", methods=["GET"])
def content_type_and_payload_mismatch() -> ResponseReturnValue:
    html = """
    <html>
    <head>
        <title>Test</title>
    </head>
    <body>
        <h1>This is HTML content</h1>
    </body>
    </html>
    """
    resp = make_response(html)
    resp.headers["Content-Type"] = "application/json"
    resp.status_code = 404
    return resp


@app.route("/api/v1/no-integrity", methods=["GET"])
def no_integrity() -> ResponseReturnValue:
    test_file_path = get_test_file_path("foo.txt")
    with open(test_file_path, "rb") as f:
        resp = Response(f.read())
        resp.content_type = "application/octet-stream"
        resp.content_length = os.path.getsize(test_file_path)
        return resp, 200


@app.route("/api/v1/corrupted", methods=["GET"])
def corrupted() -> ResponseReturnValue:
    test_file_path = get_test_file_path("foo.txt")
    with open(test_file_path, "rb") as f:
        resp = Response(f.read())
        resp.content_type = "application/octet-stream"
        resp.content_length = os.path.getsize(test_file_path)
        resp.headers["x-goog-hash"] = "md5=badhash"
        return resp, 200


@app.route("/api/v1/good", methods=["GET"])
def good() -> ResponseReturnValue:
    test_file_path = get_test_file_path("foo.txt")
    with open(test_file_path, "rb") as f:
        start = 0
        if "Range" in request.headers:
            bytes_range = request.headers["Range"]
            # See: https://developer.mozilla.org/en-US/docs/Web/HTTP/Range_requests
            # Format is bytes=<start>-<end>
            # If <end> is missing, then, it means until the end.
            # This test server only supports bytes=<start>- format since it is the only one we need.
            m = re.match("^bytes=([0-9]+)-$", bytes_range)
            if not m:
                return "", 400
            start = int(m.group(1))
        f.seek(start)
        content = f.read()
        file_hash = hashlib.md5()
        file_hash.update(content)
        return (
            Response(
                content,
                headers={
                    "Content-type": "application/octet-stream",
                    "Content-Length": str(os.path.getsize(test_file_path)),
                    "Accept-Ranges": "bytes",
                    "x-goog-hash": f"crc32c=n03x6A==, md5={to_b64_digest(file_hash)}",
                },
            ),
            200,
        )



import logging
import os
import time
from typing import Optional

from kagglehub.clients import ColabClient
from kagglehub.config import is_colab_cache_disabled
from kagglehub.exceptions import BackendError, NotFoundError
from kagglehub.handle import DatasetHandle, ModelHandle
from kagglehub.logger import EXTRA_CONSOLE_BLOCK
from kagglehub.resolver import Resolver

COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME = "COLAB_CACHE_MOUNT_FOLDER"
DEFAULT_COLAB_CACHE_MOUNT_FOLDER = "/kaggle/input"

logger = logging.getLogger(__name__)


class ModelColabCacheResolver(Resolver[ModelHandle]):
    def is_supported(self, handle: ModelHandle, *_, **__) -> bool:  # noqa: ANN002, ANN003
        if ColabClient.TBE_RUNTIME_ADDR_ENV_VAR_NAME not in os.environ or is_colab_cache_disabled():
            return False

        api_client = ColabClient()
        data = {
            "owner": handle.owner,
            "model": handle.model,
            "framework": handle.framework,
            "variation": handle.variation,
        }

        if handle.is_versioned():
            # Colab treats version as int in the request
            data["version"] = handle.version  # type: ignore

        try:
            api_client.post(data, ColabClient.IS_MODEL_SUPPORTED_PATH, handle)
        except NotFoundError:
            return False
        return True

    def __call__(self, h: ModelHandle, path: Optional[str] = None, *, force_download: Optional[bool] = False) -> str:
        if force_download:
            logger.info(
                "Ignoring `force_download` argument when running inside the Colab notebook environment.",
                extra={**EXTRA_CONSOLE_BLOCK},
            )

        api_client = ColabClient()
        data = {
            "owner": h.owner,
            "model": h.model,
            "framework": h.framework,
            "variation": h.variation,
        }
        if h.is_versioned():
            # Colab treats version as int in the request
            data["version"] = h.version  # type: ignore

        response = api_client.post(data, ColabClient.MODEL_MOUNT_PATH, h)

        if response is None:
            no_response = "No response received or response was empty."
            raise ValueError(no_response)

        if "slug" not in response:
            msg = "'slug' field missing from response"
            raise BackendError(msg)

        base_mount_path = os.getenv(COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME, DEFAULT_COLAB_CACHE_MOUNT_FOLDER)
        cached_path = f"{base_mount_path}/{response['slug']}"

        if not os.path.exists(cached_path):
            # Only print this if the model is not already mounted.
            logger.info(f"Mounting files to {cached_path}...")

        while not os.path.exists(cached_path):
            time.sleep(5)

        if path:
            cached_filepath = f"{cached_path}/{path}"
            if not os.path.exists(cached_filepath):
                msg = (
                    f"'{path}' is not present in the model files. "
                    f"You can access the other files of the attached model at '{cached_path}'"
                )
                raise ValueError(msg)
            return cached_filepath
        return cached_path


class DatasetColabCacheResolver(Resolver[DatasetHandle]):
    def is_supported(self, handle: DatasetHandle, *_, **__) -> bool:  # noqa: ANN002, ANN003
        if ColabClient.TBE_RUNTIME_ADDR_ENV_VAR_NAME not in os.environ or is_colab_cache_disabled():
            return False

        api_client = ColabClient()
        data = {
            "owner": handle.owner,
            "dataset": handle.dataset,
        }

        if handle.is_versioned():
            # Colab treats version as int in the request
            data["version"] = handle.version  # type: ignore

        try:
            api_client.post(data, ColabClient.IS_DATASET_SUPPORTED_PATH, handle)
        except NotFoundError:
            return False
        return True

    def __call__(self, h: DatasetHandle, path: Optional[str] = None, *, force_download: Optional[bool] = False) -> str:
        if force_download:
            logger.info(
                "Ignoring `force_download` argument when running inside the Colab notebook environment.",
                extra={**EXTRA_CONSOLE_BLOCK},
            )

        api_client = ColabClient()
        data = {
            "owner": h.owner,
            "dataset": h.dataset,
        }
        if h.is_versioned():
            # Colab treats version as int in the request
            data["version"] = h.version  # type: ignore

        response = api_client.post(data, ColabClient.DATASET_MOUNT_PATH, h)

        if response is None:
            no_response = "No response received or response was empty."
            raise ValueError(no_response)

        if "slug" not in response:
            msg = "'slug' field missing from response"
            raise BackendError(msg)

        base_mount_path = os.getenv(COLAB_CACHE_MOUNT_FOLDER_ENV_VAR_NAME, DEFAULT_COLAB_CACHE_MOUNT_FOLDER)
        cached_path = f"{base_mount_path}/{response['slug']}"

        if not os.path.exists(cached_path):
            # Only print this if the dataset is not already mounted.
            logger.info(f"Mounting files to {cached_path}...")

        while not os.path.exists(cached_path):
            time.sleep(5)

        if path:
            cached_filepath = f"{cached_path}/{path}"
            if not os.path.exists(cached_filepath):
                msg = (
                    f"'{path}' is not present in the dataset files. "
                    f"You can access the other files of the attached dataset at '{cached_path}'"
                )
                raise ValueError(msg)
            return cached_filepath
        return cached_path



import os
from unittest import mock

import requests

import kagglehub
from kagglehub.config import DISABLE_KAGGLE_CACHE_ENV_VAR_NAME
from kagglehub.env import KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME
from tests.fixtures import BaseTestCase

from .server_stubs import jwt_stub as stub
from .server_stubs import serv

INVALID_ARCHIVE_NOTEBOOK_HANDLE = "invalid/invalid/invalid/invalid/invalid"
VERSIONED_NOTEBOOK_HANDLE = "alexisbcook/titanic-tutorial/versions/1"
UNVERSIONED_NOTEBOOK_HANDLE = "alexisbcook/titanic-tutorial"
TEST_FILEPATH = "foo.txt"


# Test cases for the NotebookOutputKaggleCacheResolver.
class TestKaggleCacheNotebookOutputDownload(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app, KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME, "http://localhost:7778")

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_unversioned_notebook_output_download(self) -> None:
        with stub.create_env():
            notebook_path = kagglehub.notebook_output_download(UNVERSIONED_NOTEBOOK_HANDLE)
            self.assertEqual(["bar.csv", "foo.txt"], sorted(os.listdir(notebook_path)))

    def test_versioned_notebook_output_download(self) -> None:
        with stub.create_env():
            notebook_path = kagglehub.notebook_output_download(VERSIONED_NOTEBOOK_HANDLE)
            self.assertEqual(["foo.txt"], sorted(os.listdir(notebook_path)))

    def test_versioned_notebook_output_download_with_path(self) -> None:
        with stub.create_env():
            notebook_file_path = kagglehub.notebook_output_download(VERSIONED_NOTEBOOK_HANDLE, "foo.txt")
            self.assertTrue(notebook_file_path.endswith("foo.txt"))
            self.assertTrue(os.path.isfile(notebook_file_path))

    def test_unversioned_notebook_output_download_with_path(self) -> None:
        with stub.create_env():
            notebook_file_path = kagglehub.notebook_output_download(UNVERSIONED_NOTEBOOK_HANDLE, "bar.csv")
            self.assertTrue(notebook_file_path.endswith("bar.csv"))
            self.assertTrue(os.path.isfile(notebook_file_path))

    def test_versioned_notebook_output_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.notebook_output_download(VERSIONED_NOTEBOOK_HANDLE, "missing.txt")

    def test_unversioned_notebook_output_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.notebook_output_download(UNVERSIONED_NOTEBOOK_HANDLE, "missing.txt")

    def test_kaggle_resolver_skipped(self) -> None:
        with mock.patch.dict(os.environ, {DISABLE_KAGGLE_CACHE_ENV_VAR_NAME: "true"}):
            with stub.create_env():
                # Assert that a ConnectionError is set (uses HTTP server which is not set)
                with self.assertRaises(requests.exceptions.ConnectionError):
                    kagglehub.notebook_output_download(UNVERSIONED_NOTEBOOK_HANDLE)

    def test_versioned_notebook_output_download_bad_handle_raises(self) -> None:
        with self.assertRaises(ValueError):
            kagglehub.notebook_output_download("bad handle")

    def test_versioned_notebook_output_download_with_force_download(self) -> None:
        with stub.create_env():
            notebook_path = kagglehub.notebook_output_download(VERSIONED_NOTEBOOK_HANDLE)
            notebook_path_forced = kagglehub.notebook_output_download(VERSIONED_NOTEBOOK_HANDLE, force_download=True)

            # Using force_download shouldn't change the expected output of model_download.
            self.assertEqual(["foo.txt"], sorted(os.listdir(notebook_path_forced)))
            self.assertEqual(notebook_path, notebook_path_forced)

    def test_versioned_notebook_output_download_with_force_download_explicitly_false(self) -> None:
        with stub.create_env():
            notebook_path = kagglehub.notebook_output_download(VERSIONED_NOTEBOOK_HANDLE, force_download=False)
            self.assertEqual(["foo.txt"], sorted(os.listdir(notebook_path)))



import os
from unittest import mock

import requests

import kagglehub
from kagglehub.config import DISABLE_KAGGLE_CACHE_ENV_VAR_NAME
from kagglehub.env import KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME
from tests.fixtures import BaseTestCase

from .server_stubs import jwt_stub as stub
from .server_stubs import serv

INVALID_ARCHIVE_COMPETITION_HANDLE = "invalid/invalid"
COMPETITION_HANDLE = "squid-game"
TEST_FILEPATH = "foo.txt"


# Test cases for the CompetitionKaggleCacheResolver.
class TestKaggleCacheCompetitionDownload(BaseTestCase):
    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app, KAGGLE_DATA_PROXY_URL_ENV_VAR_NAME, "http://localhost:7778")

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def test_kaggle_resolver_skipped(self) -> None:
        with mock.patch.dict(os.environ, {DISABLE_KAGGLE_CACHE_ENV_VAR_NAME: "true"}):
            with stub.create_env():
                # Assert that a ConnectionError is set (uses HTTP server which is not set)
                with self.assertRaises(requests.exceptions.ConnectionError):
                    kagglehub.competition_download(COMPETITION_HANDLE)

    def test_competition_download(self) -> None:
        with stub.create_env():
            competition_path = kagglehub.competition_download(COMPETITION_HANDLE)
            self.assertEqual(["bar.csv", "foo.txt"], sorted(os.listdir(competition_path)))

    def test_competition_download_with_path(self) -> None:
        with stub.create_env():
            competition_path = kagglehub.competition_download(COMPETITION_HANDLE, "bar.csv")
            self.assertTrue(competition_path.endswith("bar.csv"))
            self.assertTrue(os.path.isfile(competition_path))

    def test_competition_download_with_missing_file_raises(self) -> None:
        with stub.create_env():
            with self.assertRaises(ValueError):
                kagglehub.competition_download(COMPETITION_HANDLE, "missing.txt")

    def test_competition_download_bad_handle_raises(self) -> None:
        with self.assertRaises(ValueError):
            kagglehub.competition_download(INVALID_ARCHIVE_COMPETITION_HANDLE)

    def test_competition_download_with_force_download(self) -> None:
        with stub.create_env():
            competition_path = kagglehub.competition_download(COMPETITION_HANDLE)
            competition_path_forced = kagglehub.competition_download(COMPETITION_HANDLE, force_download=True)
            self.assertEqual(["bar.csv", "foo.txt"], sorted(os.listdir(competition_path_forced)))
            self.assertEqual(competition_path, competition_path_forced)

    def test_competition_download_with_force_download_explicitly_false(self) -> None:
        with stub.create_env():
            competition_path = kagglehub.competition_download(COMPETITION_HANDLE, force_download=False)
            self.assertEqual(["bar.csv", "foo.txt"], sorted(os.listdir(competition_path)))



import os
from typing import Optional

import kagglehub
from kagglehub.cache import NOTEBOOKS_CACHE_SUBFOLDER, get_cached_archive_path
from kagglehub.handle import parse_notebook_handle
from tests.fixtures import BaseTestCase

from .server_stubs import notebook_output_download_stub as stub
from .server_stubs import serv
from .utils import create_test_cache

INVALID_ARCHIVE_NOTEBOOK_OUTPUT_HANDLE = "invalid/invalid/invalid/invalid/invalid"
VERSIONED_NOTEBOOK_OUTPUT_HANDLE = "khsamaha/simple-lightgbm-kaggle-sticker-sales-py/versions/2"
UNVERSIONED_NOTEBOOK_OUTPUT_HANDLE = "khsamaha/simple-lightgbm-kaggle-sticker-sales-py"
TEST_FILEPATH = "foo.txt"
TEST_CONTENTS = "foo"


EXPECTED_NOTEBOOK_SUBDIR = os.path.join(
    NOTEBOOKS_CACHE_SUBFOLDER, "khsamaha", "simple-lightgbm-kaggle-sticker-sales-py", "output", "versions", "2"
)


class TestHttpNotebookOutputDownload(BaseTestCase):

    @classmethod
    def setUpClass(cls):
        cls.server = serv.start_server(stub.app)

    @classmethod
    def tearDownClass(cls):
        cls.server.shutdown()

    def _download_notebook_output_and_assert_downloaded(
        self,
        d: str,
        notebook_handle: str,
        expected_subdir_or_subpath: str,
        expected_files: Optional[list[str]] = None,
        **kwargs,  # noqa: ANN003
    ) -> None:
        # Download the full notebook output and ensure all files are there.
        notebook_path = kagglehub.notebook_output_download(notebook_handle, **kwargs)
        self.assertEqual(os.path.join(d, expected_subdir_or_subpath), notebook_path)

        if not expected_files:
            expected_files = ["foo.txt"]
        self.assertEqual(sorted(expected_files), sorted(os.listdir(notebook_path)))

        # Assert that the archive file has been deleted
        archive_path = get_cached_archive_path(parse_notebook_handle(notebook_handle))
        self.assertFalse(os.path.exists(archive_path))

    def _download_test_file_and_assert_downloaded(self, d: str, notebook_handle: str, **kwargs) -> None:  # noqa: ANN003
        notebook_path = kagglehub.notebook_output_download(notebook_handle, path=TEST_FILEPATH, **kwargs)
        self.assertEqual(os.path.join(d, EXPECTED_NOTEBOOK_SUBDIR, TEST_FILEPATH), notebook_path)
        with open(notebook_path) as notebook_file:
            self.assertEqual(TEST_CONTENTS, notebook_file.read())

    def test_notebook_download_bad_archive(self) -> None:
        with create_test_cache():
            with self.assertRaises(ValueError):
                kagglehub.notebook_output_download(INVALID_ARCHIVE_NOTEBOOK_OUTPUT_HANDLE)

    def test_unversioned_notebook_output_download(self) -> None:
        with create_test_cache() as d:
            self._download_notebook_output_and_assert_downloaded(
                d, UNVERSIONED_NOTEBOOK_OUTPUT_HANDLE, EXPECTED_NOTEBOOK_SUBDIR
            )

    def test_versioned_notebook_output_download(self) -> None:
        with create_test_cache() as d:
            self._download_notebook_output_and_assert_downloaded(
                d, VERSIONED_NOTEBOOK_OUTPUT_HANDLE, EXPECTED_NOTEBOOK_SUBDIR
            )

    def test_versioned_dataset_download_with_path(self) -> None:
        with create_test_cache() as d:
            self._download_test_file_and_assert_downloaded(d, VERSIONED_NOTEBOOK_OUTPUT_HANDLE)



import logging
from typing import Optional

from kagglehub import registry
from kagglehub.handle import parse_competition_handle
from kagglehub.logger import EXTRA_CONSOLE_BLOCK

logger = logging.getLogger(__name__)


def competition_download(handle: str, path: Optional[str] = None, *, force_download: Optional[bool] = False) -> str:
    """Download competition dataset
    Args:
        handle: (string) the competition name
        path: (string) Optional path to a file within a competition dataset
        force_download: (bool) Optional flag to force download a competition dataset, even if it's cached
    Returns:
        A string requesting the path to the requested competition files.
    """

    h = parse_competition_handle(handle)
    logger.info(f"Downloading competition: {h.to_url()} ...", extra={**EXTRA_CONSOLE_BLOCK})
    return registry.competition_resolver(h, path, force_download=force_download)



from flask import Flask, jsonify, request
from flask.typing import ResponseReturnValue

from kagglehub.http_resolver import DATASET_CURRENT_VERSION_FIELD
from tests.utils import AUTO_COMPRESSED_FILE_NAME, add_mock_gcs_route, get_gcs_redirect_response

app = Flask(__name__)
add_mock_gcs_route(app)

TARGZ_ARCHIVE_HANDLE = "testuser/zip-dataset/versions/1"


@app.route("/", methods=["HEAD"])
def head() -> ResponseReturnValue:
    return "", 200


@app.route("/api/v1/datasets/view/<owner_slug>/<dataset_slug>", methods=["GET"])
def dataset_get(owner_slug: str, dataset_slug: str) -> ResponseReturnValue:
    data = {
        "message": f"Dataset exists {owner_slug}/{dataset_slug} !",
        DATASET_CURRENT_VERSION_FIELD: 2,
    }
    return jsonify(data), 200


# For Datasets, downloads of the archive and individual files happen at the same route, controlled
# by a file_name query param
@app.route("/api/v1/datasets/download/<owner_slug>/<dataset_slug>", methods=["GET"])
def dataset_download(owner_slug: str, dataset_slug: str) -> ResponseReturnValue:
    handle = f"{owner_slug}/{dataset_slug}"

    # First, determine if we're fetching a file or the whole dataset
    file_name_query_param = request.args.get("file_name")
    if file_name_query_param:
        # This mimics behavior for our file downloads, where users request a file, but
        # receive a zipped version of the file from GCS.
        test_file_name = (
            f"{AUTO_COMPRESSED_FILE_NAME}.zip"
            if file_name_query_param == AUTO_COMPRESSED_FILE_NAME
            else file_name_query_param
        )
    # Check a special case to handle tar.gz
    elif handle in TARGZ_ARCHIVE_HANDLE:
        test_file_name = "archive.tar.gz"
    else:
        test_file_name = "foo.txt.zip"

    return get_gcs_redirect_response(test_file_name)


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"message": "Some response data", "error": str(e)}
    return jsonify(data), 404



"""Test helper functions in kagglehub."""

import pathlib
import tempfile

from kagglehub.gcs_upload import filtered_walk, normalize_patterns
from tests.fixtures import BaseTestCase


class TesModelsHelpers(BaseTestCase):
    def testnormalize_patterns(self) -> None:
        default_patterns = [".git/", ".cache/", ".gitignore"]
        self.assertEqual(
            normalize_patterns(default=default_patterns, additional=None),
            [".git/*", ".cache/*", ".gitignore"],
        )
        self.assertEqual(
            normalize_patterns(default=default_patterns, additional=["original/", "*/*.txt", "doc/readme.txt"]),
            [".git/*", ".cache/*", ".gitignore", "original/*", "*/*.txt", "doc/readme.txt"],
        )

    def test_filtered_walk(self) -> None:
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_dir_p = pathlib.Path(tmp_dir)

            # files to upload
            (tmp_dir_p / "a" / "b").mkdir(parents=True)
            (tmp_dir_p / "weights.txt").touch()
            (tmp_dir_p / "a" / "a.txt").touch()
            (tmp_dir_p / "a" / "b" / "b.txt").touch()
            (tmp_dir_p / "a" / "b" / ".bb").touch()
            expected_files = {
                tmp_dir_p / "weights.txt",
                tmp_dir_p / "a" / "a.txt",
                tmp_dir_p / "a" / "b" / "b.txt",
                tmp_dir_p / "a" / "b" / ".bb",
            }

            # files to ignore
            (tmp_dir_p / ".git").mkdir(parents=True)
            (tmp_dir_p / ".git" / "file").write_text("hidden git file")
            (tmp_dir_p / ".gitignore").write_text("none")

            (tmp_dir_p / "a" / ".git").mkdir(parents=True)
            (tmp_dir_p / "a" / "b" / ".git").mkdir(parents=True)
            (tmp_dir_p / "a" / "b" / ".git" / "abgit.txt").write_text("abgit")

            (tmp_dir_p / "a" / "b" / ".hidden").touch()

            (tmp_dir_p / "original" / "fp8").mkdir(parents=True)
            (tmp_dir_p / "original" / "fp8" / "weights").touch()
            (tmp_dir_p / "original" / "fp16").mkdir(parents=True)
            (tmp_dir_p / "original" / "fp16" / "weights").touch()

            # filtered walk
            ignore_patterns = normalize_patterns(
                default=[".git/", "*/.git/", ".gitignore", "*/.hidden", "original/"], additional=None
            )
            walked_files = []
            for dir_path, _, file_names in filtered_walk(base_dir=tmp_dir, ignore_patterns=ignore_patterns):
                for file_name in file_names:
                    walked_files.append(pathlib.Path(dir_path) / file_name)
            self.assertEqual(set(walked_files), expected_files)



from flask import Flask, jsonify, request
from flask.typing import ResponseReturnValue

from kagglehub.http_resolver import NOTEBOOK_CURRENT_VERSION_FIELD
from tests.utils import (
    AUTO_COMPRESSED_FILE_NAME,
    add_mock_gcs_route,
    get_gcs_redirect_response,
)

app = Flask(__name__)
add_mock_gcs_route(app)

# See https://cloud.google.com/storage/docs/xml-api/reference-headers#xgooghash
GCS_HASH_HEADER = "x-goog-hash"
LAST_MODIFIED = "Last-Modified"
LAST_MODIFIED_DATE = "Thu, 02 Mar 2020 02:17:12 GMT"


@app.route("/", methods=["HEAD"])
def head() -> ResponseReturnValue:
    return "", 200


@app.route("/api/v1/kernels/pull", methods=["GET"])
def notebook_get() -> ResponseReturnValue:
    user_name = request.args.get("user_name")
    kernel_slug = request.args.get("kernel_slug")
    data = {"message": f"Notebook exists {user_name}/{kernel_slug} !", "metadata": {NOTEBOOK_CURRENT_VERSION_FIELD: 2}}
    return jsonify(data), 200


@app.route("/api/v1/kernels/output/download/<owner_slug>/<kernel_slug>", methods=["GET"])
def notebook_output_download(owner_slug: str, kernel_slug: str) -> ResponseReturnValue:
    # Name to prevent linter from complaining about unused variables.
    _ = f"{owner_slug}/{kernel_slug}"

    # First, determine if we're fetching a file or the whole notebook output
    file_name_query_param = request.args.get("file_path")
    test_file_name = "foo.txt.zip"
    if file_name_query_param:
        # This mimics behavior for our file downloads, where users request a file, but
        # receive a zipped version of the file from GCS.
        test_file_name = (
            f"{AUTO_COMPRESSED_FILE_NAME}.zip"
            if file_name_query_param == AUTO_COMPRESSED_FILE_NAME
            else file_name_query_param
        )

    return get_gcs_redirect_response(test_file_name)


@app.route("/api/v1/kernels/output/list/<owner_slug>/<kernel_slug>", methods=["GET"])
def notebook_list_files(owner_slug: str, kernel_slug: str) -> ResponseReturnValue:
    # Name to prevent linter from complaining about unused variables.
    _ = f"{owner_slug}/{kernel_slug}"

    data = {"files": [{"url": "testUrl", "fileName": "foo.txt"}]}

    return jsonify(data), 200


@app.errorhandler(404)
def error(e: Exception):  # noqa: ANN201
    data = {"message": "Some error response data", "error": str(e)}
    return jsonify(data), 404
